/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;

import com.tinkerforge.BrickStepper;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Stepper Brick.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickStepperWrapper extends BrickStepper implements DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickStepper.DEVICE_DISPLAY_NAME, "brickstepper", BrickStepper.DEVICE_IDENTIFIER, BrickStepperWrapper.class, BrickStepperActions.class, "2.3.6", false);

    public BrickStepperWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickStepperWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickStepperConfig cfg = (BrickStepperConfig) config.as(BrickStepperConfig.class);
        this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperCurrentVelocity", transformBrickStepperCurrentVelocityCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperCurrentPosition", transformBrickStepperCurrentPositionCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperRemainingSteps", transformBrickStepperRemainingStepsCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperStackVoltage", transformBrickStepperStackVoltageCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperExternalVoltage", transformBrickStepperExternalVoltageCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickStepperCurrentConsumption", transformBrickStepperCurrentConsumptionCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addNewStateListener(this.reg((stateNew, statePrevious) -> {if(true) {updateStateFn.accept("BrickStepperState", transformBrickStepperStateCallback0(stateNew, statePrevious, cfg));}}, this::removeNewStateListener));
		this.addNewStateListener(this.reg((stateNew, statePrevious) -> {if(true) {updateStateFn.accept("BrickStepperPreviousState", transformBrickStepperPreviousStateCallback0(stateNew, statePrevious, cfg));}}, this::removeNewStateListener));
		this.addPositionReachedListener(this.reg((position) -> {if(true) {triggerChannelFn.accept("BrickStepperPositionReached", transformBrickStepperPositionReachedCallback0(position, cfg));}}, this::removePositionReachedListener));
		this.addUnderVoltageListener(this.reg((voltage) -> {if(true) {triggerChannelFn.accept("BrickStepperUnterVoltage", transformBrickStepperUnterVoltageCallback0(voltage, cfg));}}, this::removeUnderVoltageListener));
        this.setAllDataPeriod(cfg.allDataUpdateInterval);
		    this.setMinimumVoltage((int)(cfg.minimumVoltage.doubleValue() * 1000.0));
		if(cfg.statusLEDConfig) {
		                this.enableStatusLED();
		            } else {
		                this.disableStatusLED();
		            }
		this.setSPITFPBaudrateConfig(cfg.spitfpEnableDynamicBaudrate, cfg.spitfpMinimumDynamicBaudrate);
		this.setSPITFPBaudrate('a', cfg.spitfpBaudratePortA);
		            this.setSPITFPBaudrate('b', cfg.spitfpBaudratePortB);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickStepperCurrentVelocity");
		result.add("BrickStepperCurrentPosition");
		result.add("BrickStepperRemainingSteps");
		result.add("BrickStepperStackVoltage");
		result.add("BrickStepperExternalVoltage");
		result.add("BrickStepperCurrentConsumption");
		result.add("BrickStepperState");
		result.add("BrickStepperPreviousState");
		result.add("BrickStepperPositionReached");
		result.add("BrickStepperUnterVoltage");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickStepperCurrentVelocity":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperCurrentVelocity"), "Velocity", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperCurrentVelocity")).withDescription("The current velocity of the stepper motor in steps per second.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperCurrentPosition":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperCurrentPosition"), "Position", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperCurrentPosition")).withDescription("The current position of the stepper motor in steps. On startup the position is 0.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperRemainingSteps":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperRemainingSteps"), "Remaining Steps", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperRemainingSteps")).withDescription("The remaining steps of the last call of the setSteps() action.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperStackVoltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperStackVoltage"), "Stack Voltage", "Number:ElectricPotential").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperStackVoltage")).withDescription("The stack input voltage in V. The stack input voltage is the voltage that is supplied via the stack, i.e. it is given by a Step-Down or Step-Up Power Supply.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperExternalVoltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperExternalVoltage"), "External Voltage", "Number:ElectricPotential").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperExternalVoltage")).withDescription("The external input voltage in mV. The external input voltage is given via the black power input connector on the Stepper Brick.<br/><br/>If there is an external input voltage and a stack input voltage, the motor will be driven by the external input voltage. If there is only a stack voltage present, the motor will be driven by this voltage.<br/><br/><b>Warning: This means, if you have a high stack voltage and a low external voltage, the motor will be driven with the low external voltage. If you then remove the external connection, it will immediately be driven by the high stack voltage</b>")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperCurrentConsumption":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperCurrentConsumption"), "Current Consumption", "Number:ElectricCurrent").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperCurrentConsumption")).withDescription("The current consumption of the motor in A.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickStepperState":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickStepperState"), "State", "Number").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickStepperState")).withDescription("State of the brick: <ul><li>Stop = 1</li><li>Acceleration = 2</li><li>Run = 3</li><li>Deacceleration = 4</li><li>Direction change to forward = 5</li><li>Direction change to backward = 6</li></ul>")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Stepper Brick").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Drives one bipolar stepper motor with up to 38V and 2.5A per phase").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickStepperCurrentVelocity", new ChannelTypeUID("tinkerforge", "BrickStepperCurrentVelocity")).withLabel("Velocity").build(), new ChannelDefinitionBuilder("BrickStepperCurrentPosition", new ChannelTypeUID("tinkerforge", "BrickStepperCurrentPosition")).withLabel("Position").build(), new ChannelDefinitionBuilder("BrickStepperRemainingSteps", new ChannelTypeUID("tinkerforge", "BrickStepperRemainingSteps")).withLabel("Remaining Steps").build(), new ChannelDefinitionBuilder("BrickStepperStackVoltage", new ChannelTypeUID("tinkerforge", "BrickStepperStackVoltage")).withLabel("Stack Voltage").build(), new ChannelDefinitionBuilder("BrickStepperExternalVoltage", new ChannelTypeUID("tinkerforge", "BrickStepperExternalVoltage")).withLabel("External Voltage").build(), new ChannelDefinitionBuilder("BrickStepperCurrentConsumption", new ChannelTypeUID("tinkerforge", "BrickStepperCurrentConsumption")).withLabel("Current Consumption").build(), new ChannelDefinitionBuilder("BrickStepperState", new ChannelTypeUID("tinkerforge", "BrickStepperState")).withLabel("State").build(), new ChannelDefinitionBuilder("BrickStepperPreviousState", new ChannelTypeUID("tinkerforge", "BrickStepperState")).withLabel("Previous State").build(), new ChannelDefinitionBuilder("BrickStepperPositionReached", new ChannelTypeUID("system", "trigger")).withLabel("Position Reached").withDescription("This channel is triggered when a position set by the setSteps or setTargetPosition action is reached.<br/><br/>Note<br/><br/>Since we can't get any feedback from the stepper motor, this only works if the acceleration is set smaller or equal to the maximum acceleration of the motor. Otherwise the motor will lag behind the control value and the listener will be triggered too early.").build(), new ChannelDefinitionBuilder("BrickStepperUnterVoltage", new ChannelTypeUID("system", "trigger")).withLabel("Unter Voltage").withDescription("This channel is triggered when the input voltage drops below the configured minimum voltage.").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickstepper":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("allDataUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for all data in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("All Data Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("minimumVoltage", Type.DECIMAL).withDefault("8").withDescription("The minimum voltage in V, below which the Unter Voltage channel is triggered. The minimum possible value that works with the Stepper Brick is 8V. You can use this function to detect the discharge of a battery that is used to drive the stepper motor. If you have a fixed power supply, you likely do not need this functionality. The default value is 8V.").withLabel("Minimum Voltage").withUnit("V").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(65.535)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.BOOLEAN).withDefault("true").withDescription("The status LED is the blue LED next to the USB connector. If enabled is is on and it flickers if data is transfered. If disabled it is always off.").withLabel("Status LED Configuration").build(), ConfigDescriptionParameterBuilder.create("spitfpEnableDynamicBaudrate", Type.BOOLEAN).withDefault("true").withDescription("The SPITF protocol can be used with a dynamic baudrate. If the dynamic baudrate is enabled, the Brick will try to adapt the baudrate for the communication between Bricks and Bricklets according to the amount of data that is transferred.<br/><br/>The baudrate will be increased exponentially if lots of data is sent/received and decreased linearly if little data is sent/received.<br/><br/>This lowers the baudrate in applications where little data is transferred (e.g. a weather station) and increases the robustness. If there is lots of data to transfer (e.g. Thermal Imaging Bricklet) it automatically increases the baudrate as needed.<br/><br/>In cases where some data has to transferred as fast as possible every few seconds (e.g. RS485 Bricklet with a high baudrate but small payload) you may want to turn the dynamic baudrate off to get the highest possible performance.<br/><br/>The maximum value of the baudrate can be set per port. If the dynamic baudrate is disabled, the maximum baudrate will be used statically.").withLabel("SPITFP Enable Dynamic Baudrate").withAdvanced(true).build(), ConfigDescriptionParameterBuilder.create("spitfpMinimumDynamicBaudrate", Type.INTEGER).withDefault("400000").withDescription("See SPITFP Enable Dynamic Baudrate").withLabel("SPITFP Minimum Dynamic Baudrate").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("spitfpBaudratePortA", Type.INTEGER).withDefault("1400000").withDescription("The baudrate for Bricklet port A.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in your applications we recommend to not change the baudrate.").withLabel("(Maximum) SPITFP Baudrate Port A").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("spitfpBaudratePortB", Type.INTEGER).withDefault("1400000").withDescription("The baudrate for Bricklet port B.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in your applications we recommend to not change the baudrate.").withLabel("(Maximum) SPITFP Baudrate Port B").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickStepperCurrentVelocity":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperCurrentPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperRemainingSteps":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperStackVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperExternalVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperCurrentConsumption":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickStepperState":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickStepperConfig cfg = (BrickStepperConfig) config.as(BrickStepperConfig.class);
switch(channel) {
            case "BrickStepperCurrentVelocity":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperCurrentVelocityGetter0(value, cfg));
        }
                   return;
            case "BrickStepperCurrentPosition":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperCurrentPositionGetter0(value, cfg));
        }
                   return;
            case "BrickStepperRemainingSteps":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperRemainingStepsGetter0(value, cfg));
        }
                   return;
            case "BrickStepperStackVoltage":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperStackVoltageGetter0(value, cfg));
        }
                   return;
            case "BrickStepperExternalVoltage":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperExternalVoltageGetter0(value, cfg));
        }
                   return;
            case "BrickStepperCurrentConsumption":
                   {
            BrickStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickStepperCurrentConsumptionGetter0(value, cfg));
        }
                   return;
            case "BrickStepperState":
               return;
            case "BrickStepperPreviousState":
               return;
            case "BrickStepperPositionReached":
               return;
            case "BrickStepperUnterVoltage":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentVelocityCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new DecimalType(currentVelocity);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentPositionCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new DecimalType(currentPosition);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperRemainingStepsCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new DecimalType(remainingSteps);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperStackVoltageCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new QuantityType<>(stackVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperExternalVoltageCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new QuantityType<>(externalVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentConsumptionCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickStepperConfig cfg) {
        return new QuantityType<>(currentConsumption / 1000.0, SmartHomeUnits.AMPERE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperStateCallback0(short stateNew, short statePrevious, BrickStepperConfig cfg) {
        return new DecimalType(stateNew);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperPreviousStateCallback0(short stateNew, short statePrevious, BrickStepperConfig cfg) {
        return new DecimalType(statePrevious);
    }
	    private String transformBrickStepperPositionReachedCallback0(int position, BrickStepperConfig cfg) {
        return "";
    }
	    private String transformBrickStepperUnterVoltageCallback0(int voltage, BrickStepperConfig cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentVelocityGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new DecimalType(value.currentVelocity);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentPositionGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new DecimalType(value.currentPosition);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperRemainingStepsGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new DecimalType(value.remainingSteps);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperStackVoltageGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new QuantityType<>(value.stackVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperExternalVoltageGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new QuantityType<>(value.externalVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickStepperCurrentConsumptionGetter0(BrickStepper.AllData value, BrickStepperConfig cfg) {
        return new QuantityType<>(value.currentConsumption / 1000.0, SmartHomeUnits.AMPERE);
    }

}