/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OnOffType;

import com.tinkerforge.BrickletAccelerometer;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Accelerometer Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletAccelerometerWrapper extends BrickletAccelerometer implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletAccelerometer.DEVICE_DISPLAY_NAME, "brickletaccelerometer", BrickletAccelerometer.DEVICE_IDENTIFIER, BrickletAccelerometerWrapper.class, BrickletAccelerometerActions.class, "2.0.0", false);

    public BrickletAccelerometerWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletAccelerometerWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletAccelerometerConfig cfg = (BrickletAccelerometerConfig) config.as(BrickletAccelerometerConfig.class);
        this.addAccelerationListener(this.reg((x, y, z) -> {if(true) {updateStateFn.accept("BrickletAccelerometerAccelerationX", transformBrickletAccelerometerAccelerationXCallback0(x, y, z, cfg));}}, this::removeAccelerationListener));
		this.addAccelerationListener(this.reg((x, y, z) -> {if(true) {updateStateFn.accept("BrickletAccelerometerAccelerationY", transformBrickletAccelerometerAccelerationYCallback0(x, y, z, cfg));}}, this::removeAccelerationListener));
		this.addAccelerationListener(this.reg((x, y, z) -> {if(true) {updateStateFn.accept("BrickletAccelerometerAccelerationZ", transformBrickletAccelerometerAccelerationZCallback0(x, y, z, cfg));}}, this::removeAccelerationListener));
        this.setConfiguration(cfg.dataRate.shortValue(), cfg.fullScaleRange.shortValue(), cfg.filterBandwidth.shortValue());
		    this.setAccelerationCallbackPeriod(cfg.accelerationUpdateInterval);
		{
		BrickletAccelerometerTemperatureConfig channelCfg = getChannelConfigFn.apply("BrickletAccelerometerTemperature").as(BrickletAccelerometerTemperatureConfig.class);
		
		if(channelCfg.updateInterval > 0) {
    this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {if(handler.getThing().getStatus() == ThingStatus.ONLINE) {handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletAccelerometerTemperature"), RefreshType.REFRESH);}}, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
}
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletAccelerometerAccelerationX");
		result.add("BrickletAccelerometerAccelerationY");
		result.add("BrickletAccelerometerAccelerationZ");
		result.add("BrickletAccelerometerTemperature");
		result.add("BrickletAccelerometerLED");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletAccelerometerAcceleration":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletAccelerometerAcceleration"), "NOT USED", "Number:Acceleration").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAccelerometerAcceleration")).withDescription("The acceleration in g (1g = 9.80665m/s²), not to be confused with grams.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletAccelerometerLED":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletAccelerometerLED"), "LED", "Switch").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAccelerometerLED")).withDescription("Enables/Disables the LED on the Bricklet.").build();
            case "BrickletAccelerometerTemperature":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletAccelerometerTemperature"), "Temperature", "Number:Temperature").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAccelerometerTemperature")).withDescription("The temperature of the accelerometer")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-103)).withMaximum(BigDecimal.valueOf(152)).withStep(BigDecimal.valueOf(1)).withPattern("%d %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Accelerometer Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Measures acceleration in three axis").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletAccelerometerAccelerationX", new ChannelTypeUID("tinkerforge", "BrickletAccelerometerAcceleration")).withLabel("Acceleration X").build(), new ChannelDefinitionBuilder("BrickletAccelerometerAccelerationY", new ChannelTypeUID("tinkerforge", "BrickletAccelerometerAcceleration")).withLabel("Acceleration Y").build(), new ChannelDefinitionBuilder("BrickletAccelerometerAccelerationZ", new ChannelTypeUID("tinkerforge", "BrickletAccelerometerAcceleration")).withLabel("Acceleration Z").build(), new ChannelDefinitionBuilder("BrickletAccelerometerTemperature", new ChannelTypeUID("tinkerforge", "BrickletAccelerometerTemperature")).withLabel("Temperature").build(), new ChannelDefinitionBuilder("BrickletAccelerometerLED", new ChannelTypeUID("tinkerforge", "BrickletAccelerometerLED")).withLabel("LED").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletaccelerometer":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("dataRate", Type.INTEGER).withDefault("6").withDescription("The data rate of 0Hz to 1600Hz. Decreasing data rate or full scale range will also decrease the noise on the data.").withLabel("Data Rate").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "3Hz"), new ParameterOption("2", "6Hz"), new ParameterOption("3", "12Hz"), new ParameterOption("4", "25Hz"), new ParameterOption("5", "50Hz"), new ParameterOption("6", "100Hz"), new ParameterOption("7", "400Hz"), new ParameterOption("8", "800Hz"), new ParameterOption("9", "1600Hz"))).build(), ConfigDescriptionParameterBuilder.create("fullScaleRange", Type.INTEGER).withDefault("1").withDescription("Full scale range of ±2gₙ up to ±16gₙ. Decreasing data rate or full scale range will also decrease the noise on the data.").withLabel("Full Scale Range").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "2g"), new ParameterOption("1", "4g"), new ParameterOption("2", "6g"), new ParameterOption("3", "8g"), new ParameterOption("4", "16g"))).build(), ConfigDescriptionParameterBuilder.create("filterBandwidth", Type.INTEGER).withDefault("2").withDescription("Filter bandwidth between 50Hz and 800Hz.").withLabel("Filter Bandwidth").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "800Hz"), new ParameterOption("1", "400Hz"), new ParameterOption("2", "200Hz"), new ParameterOption("3", "50Hz"))).build(), ConfigDescriptionParameterBuilder.create("accelerationUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for the acceleration in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Acceleration Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletAccelerometerAcceleration":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletAccelerometerLED":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletAccelerometerTemperature":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withLabel("Update Interval").withUnit("ms").build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletAccelerometerConfig cfg = (BrickletAccelerometerConfig) config.as(BrickletAccelerometerConfig.class);
switch(channel) {
            case "BrickletAccelerometerAccelerationX":
                   {
            BrickletAccelerometer.Acceleration value = this.getAcceleration();
            if(true)
                updateStateFn.accept(channel, transformBrickletAccelerometerAccelerationXGetter0(value, cfg));
        }
                   return;
            case "BrickletAccelerometerAccelerationY":
                   {
            BrickletAccelerometer.Acceleration value = this.getAcceleration();
            if(true)
                updateStateFn.accept(channel, transformBrickletAccelerometerAccelerationYGetter0(value, cfg));
        }
                   return;
            case "BrickletAccelerometerAccelerationZ":
                   {
            BrickletAccelerometer.Acceleration value = this.getAcceleration();
            if(true)
                updateStateFn.accept(channel, transformBrickletAccelerometerAccelerationZGetter0(value, cfg));
        }
                   return;
            case "BrickletAccelerometerTemperature":
                   {
            short value = this.getTemperature();
            if(true)
                updateStateFn.accept(channel, transformBrickletAccelerometerTemperatureGetter0(value, cfg));
        }
                   return;
            case "BrickletAccelerometerLED":
                   {
            boolean value = this.isLEDOn();
            if(true)
                updateStateFn.accept(channel, transformBrickletAccelerometerLEDGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletAccelerometerLED":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
if(cmd == OnOffType.ON) {
    this.ledOn();
}
                }
        
if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
if(cmd == OnOffType.OFF) {
    this.ledOff();
}
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType, OnOffType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationXCallback0(short x, short y, short z, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(x / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationYCallback0(short x, short y, short z, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(y / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationZCallback0(short x, short y, short z, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(z / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationXGetter0(BrickletAccelerometer.Acceleration value, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(value.x / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationYGetter0(BrickletAccelerometer.Acceleration value, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(value.y / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerAccelerationZGetter0(BrickletAccelerometer.Acceleration value, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(value.z / 1000.0, SmartHomeUnits.STANDARD_GRAVITY);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerTemperatureGetter0(short value, BrickletAccelerometerConfig cfg) {
        return new QuantityType<>(value, SIUnits.CELSIUS);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletAccelerometerLEDGetter0(boolean value, BrickletAccelerometerConfig cfg) {
        return value? OnOffType.ON : OnOffType.OFF;
    }

}