/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import java.math.BigDecimal;

import com.tinkerforge.BrickletBarometer;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Barometer Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletBarometerWrapper extends BrickletBarometer implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletBarometer.DEVICE_DISPLAY_NAME, "brickletbarometer", BrickletBarometer.DEVICE_IDENTIFIER, BrickletBarometerWrapper.class, BrickletBarometerActions.class, "2.0.3", false);

    public BrickletBarometerWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletBarometerWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletBarometerConfig cfg = (BrickletBarometerConfig) config.as(BrickletBarometerConfig.class);
        this.addAirPressureListener(this.reg((airPressure) -> {if(true) {updateStateFn.accept("BrickletBarometerAirPressure", transformBrickletBarometerAirPressureCallback0(airPressure, cfg));}}, this::removeAirPressureListener));
		this.addAltitudeListener(this.reg((altitude) -> {if(true) {updateStateFn.accept("BrickletBarometerAltitude", transformBrickletBarometerAltitudeCallback0(altitude, cfg));}}, this::removeAltitudeListener));
        this.setReferenceAirPressure(cfg.referenceAirPressure.multiply(new BigDecimal(1000)).intValue());
		this.setAveraging(cfg.pressureMovingAverageLength.shortValue(), cfg.pressureAverageLength.shortValue(), cfg.temperatureAverageLength.shortValue());
		this.setI2CMode(cfg.i2cMode.shortValue());
		{
		BrickletBarometerAirPressureConfig channelCfg = getChannelConfigFn.apply("BrickletBarometerAirPressure").as(BrickletBarometerAirPressureConfig.class);
		this.setAirPressureCallbackPeriod(channelCfg.updateInterval);
		this.setAirPressureCallbackThreshold('x', 0, 0);
		}
		{
		BrickletBarometerAltitudeConfig channelCfg = getChannelConfigFn.apply("BrickletBarometerAltitude").as(BrickletBarometerAltitudeConfig.class);
		this.setAltitudeCallbackPeriod(channelCfg.updateInterval);
		this.setAltitudeCallbackThreshold('x', 0, 0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setAirPressureCallbackPeriod(0);
		this.setAltitudeCallbackPeriod(0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletBarometerAirPressure");
		result.add("BrickletBarometerAltitude");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletBarometerAirPressure":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletBarometerAirPressure"), "Air Pressure", "Number:Pressure").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletBarometerAirPressure")).withDescription("The measured air pressure")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0.01)).withMaximum(BigDecimal.valueOf(1.2)).withStep(BigDecimal.valueOf(1e-06)).withPattern("%.6f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletBarometerAltitude":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletBarometerAltitude"), "Altitude", "Number:Length").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletBarometerAltitude")).withDescription("The relative altitude. The value is calculated based on the difference between the current air pressure and the reference air pressure.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Barometer Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Measures air pressure and altitude changes").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletBarometerAirPressure", new ChannelTypeUID("tinkerforge", "BrickletBarometerAirPressure")).withLabel("Air Pressure").build(), new ChannelDefinitionBuilder("BrickletBarometerAltitude", new ChannelTypeUID("tinkerforge", "BrickletBarometerAltitude")).withLabel("Altitude").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletbarometer":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("referenceAirPressure", Type.DECIMAL).withDefault("1013.25").withDescription("The reference air pressure in hPa for the altitude calculation. Valid values are between 10 and 1200. Setting the reference to the current air pressure results in a calculated altitude of 0 cm.").withLabel("Reference Air Pressure").withMinimum(BigDecimal.valueOf(10)).withMaximum(BigDecimal.valueOf(1200)).build(), ConfigDescriptionParameterBuilder.create("pressureMovingAverageLength", Type.INTEGER).withDefault("25").withDescription("The length of the moving average for the air pressure. The moving average is calculated over the normal averages.").withGroupName("average").withLabel("Pressure Moving Average Length").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(25)).build(), ConfigDescriptionParameterBuilder.create("pressureAverageLength", Type.INTEGER).withDefault("10").withDescription("The number of samples to average over for the air pressure.").withGroupName("average").withLabel("Pressure Average Length").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(10)).build(), ConfigDescriptionParameterBuilder.create("temperatureAverageLength", Type.INTEGER).withDefault("10").withDescription("The number of samples to average over for the temperature.").withGroupName("average").withLabel("Temperature Average Length").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(255)).build(), ConfigDescriptionParameterBuilder.create("i2cMode", Type.INTEGER).withDefault("0").withDescription("Possible modes are:<ul><li>0: Fast (400kHz)</li><li>1: Slow (100kHz)</li><br/><br/>If you have problems with obvious outliers in the Barometer Bricklet measurements, they may be caused by EMI issues. In this case it may be helpful to lower the I2C speed.<br/><br/>It is however not recommended to lower the I2C speed in applications where a high throughput needs to be achieved.").withLabel("I2C Mode").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Fast"), new ParameterOption("1", "Slow"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"), new ConfigDescriptionParameterGroup("average", "null", true, "Averaging", "The averaging parameters. It is possible to set the length of a normal averaging for the temperature and pressure, as well as an additional length of a moving average for the pressure. The moving average is calculated from the normal averages. There is no moving average for the temperature.<br/><br/>The maximum length for the pressure average is 10, for the temperature average is 255 and for the moving average is 25.<br/><br/>Setting the all three parameters to 0 will turn the averaging completely off. If the averaging is off, there is lots of noise on the data, but the data is without delay. Thus we recommend to turn the averaging off if the Barometer Bricklet data is to be used for sensor fusion with other sensors."))).build();
            case "channel-type:tinkerforge:BrickletBarometerAirPressure":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletBarometerAltitude":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletBarometerConfig cfg = (BrickletBarometerConfig) config.as(BrickletBarometerConfig.class);
switch(channel) {
            case "BrickletBarometerAirPressure":
                   {
            int value = this.getAirPressure();
            if(true)
                updateStateFn.accept(channel, transformBrickletBarometerAirPressureGetter0(value, cfg));
        }
                   return;
            case "BrickletBarometerAltitude":
                   {
            int value = this.getAltitude();
            if(true)
                updateStateFn.accept(channel, transformBrickletBarometerAltitudeGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletBarometerAirPressureCallback0(int airPressure, BrickletBarometerConfig cfg) {
        return new QuantityType<>(airPressure / 1000000.0, SmartHomeUnits.BAR);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletBarometerAltitudeCallback0(int altitude, BrickletBarometerConfig cfg) {
        return new QuantityType<>(altitude / 100.0, SIUnits.METRE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletBarometerAirPressureGetter0(int value, BrickletBarometerConfig cfg) {
        return new QuantityType<>(value / 1000000.0, SmartHomeUnits.BAR);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletBarometerAltitudeGetter0(int value, BrickletBarometerConfig cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.METRE);
    }

}