/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletCO2V2;

/**
 * Passes through actions of the CO2 Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCO2V2Actions implements ThingActions, IBrickletCO2V2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletCO2V2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletCO2V2Actions.class.getName())) {
            if (actions instanceof IBrickletCO2V2Actions) {
                return (IBrickletCO2V2Actions) actions;
            } else {
                return (IBrickletCO2V2Actions) Proxy.newProxyInstance(IBrickletCO2V2Actions.class.getClassLoader(),
                        new Class[] { IBrickletCO2V2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletCO2V2Actions");
    }

    
    @Override
    @RuleAction(label = "Get All Values")
    public @ActionOutput(name = "co2Concentration", type="int")
           @ActionOutput(name = "temperature", type="int")
           @ActionOutput(name = "humidity", type="int")
           Map<String, Object> brickletCO2V2GetAllValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletCO2V2.AllValues value = dev.getAllValues();
        
        result.put("co2Concentration", value.co2Concentration);
        result.put("temperature", value.temperature);
        result.put("humidity", value.humidity);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetAllValues(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetAllValues();
    }

    @Override
    @RuleAction(label = "Set Air Pressure")
    public void brickletCO2V2SetAirPressure(
            @ActionInput(name = "airPressure") int airPressure) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setAirPressure(airPressure);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletCO2V2AirPressure")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletCO2V2AirPressure"), RefreshType.REFRESH);
}
    }

    public static void brickletCO2V2SetAirPressure(@Nullable ThingActions actions, int airPressure) throws TinkerforgeException {
        invokeMethodOf(actions).brickletCO2V2SetAirPressure(airPressure);
    }


    @Override
    @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletCO2V2GetAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getAirPressure();
        
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetAirPressure();
    }


    @Override
    @RuleAction(label = "Get Temperature Offset")
    public @ActionOutput(name = "offset", type="int")
           Map<String, Object> brickletCO2V2GetTemperatureOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperatureOffset();
        
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetTemperatureOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetTemperatureOffset();
    }


    @Override
    @RuleAction(label = "Get CO2 Concentration")
    public @ActionOutput(name = "co2Concentration", type="int")
           Map<String, Object> brickletCO2V2GetCO2Concentration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getCO2Concentration();
        
        result.put("co2Concentration", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetCO2Concentration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetCO2Concentration();
    }


    @Override
    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletCO2V2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetTemperature();
    }


    @Override
    @RuleAction(label = "Get Humidity")
    public @ActionOutput(name = "humidity", type="int")
           Map<String, Object> brickletCO2V2GetHumidity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getHumidity();
        
        result.put("humidity", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetHumidity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetHumidity();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletCO2V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletCO2V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletCO2V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletCO2V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletCO2V2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletCO2V2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletCO2V2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletCO2V2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletCO2V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCO2V2Wrapper dev = ((@Nullable BrickletCO2V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletCO2V2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCO2V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCO2V2GetIdentity();
    }
}
