/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletColorV2;

/**
 * Passes through actions of the Color Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletColorV2Actions implements ThingActions, IBrickletColorV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletColorV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletColorV2Actions.class.getName())) {
            if (actions instanceof IBrickletColorV2Actions) {
                return (IBrickletColorV2Actions) actions;
            } else {
                return (IBrickletColorV2Actions) Proxy.newProxyInstance(IBrickletColorV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletColorV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletColorV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Color")
    public @ActionOutput(name = "r", type="int")
           @ActionOutput(name = "g", type="int")
           @ActionOutput(name = "b", type="int")
           @ActionOutput(name = "c", type="int")
           Map<String, Object> brickletColorV2GetColor(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletColorV2.Color value = dev.getColor();
        
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("c", value.c);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetColor(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetColor();
    }


    @Override
    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type="long")
           Map<String, Object> brickletColorV2GetIlluminance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getIlluminance();
        
        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetIlluminance(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetIlluminance();
    }


    @Override
    @RuleAction(label = "Get Color Temperature")
    public @ActionOutput(name = "colorTemperature", type="int")
           Map<String, Object> brickletColorV2GetColorTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getColorTemperature();
        
        result.put("colorTemperature", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetColorTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetColorTemperature();
    }

    @Override
    @RuleAction(label = "Set Light")
    public void brickletColorV2SetLight(
            @ActionInput(name = "enable") boolean enable) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setLight(enable);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletColorV2Light")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletColorV2Light"), RefreshType.REFRESH);
}
    }

    public static void brickletColorV2SetLight(@Nullable ThingActions actions, boolean enable) throws TinkerforgeException {
        invokeMethodOf(actions).brickletColorV2SetLight(enable);
    }


    @Override
    @RuleAction(label = "Get Light")
    public @ActionOutput(name = "enable", type="boolean")
           Map<String, Object> brickletColorV2GetLight(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean value = dev.getLight();
        
        result.put("enable", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetLight(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetLight();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "gain", type="int")
           @ActionOutput(name = "integrationTime", type="int")
           Map<String, Object> brickletColorV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletColorV2.Configuration value = dev.getConfiguration();
        
        result.put("gain", value.gain);
        result.put("integrationTime", value.integrationTime);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetConfiguration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletColorV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletColorV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletColorV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletColorV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletColorV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletColorV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletColorV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletColorV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletColorV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletColorV2Wrapper dev = ((@Nullable BrickletColorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletColorV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletColorV2GetIdentity();
    }
}
