/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDMX;

/**
 * Passes through actions of the DMX Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDMXActions implements ThingActions, IBrickletDMXActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletDMXActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletDMXActions.class.getName())) {
            if (actions instanceof IBrickletDMXActions) {
                return (IBrickletDMXActions) actions;
            } else {
                return (IBrickletDMXActions) Proxy.newProxyInstance(IBrickletDMXActions.class.getClassLoader(),
                        new Class[] { IBrickletDMXActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletDMXActions");
    }

        @Override
    @RuleAction(label = "Write Frame")
    public void brickletDMXWriteFrame(
            @ActionInput(name = "frame") int[] frame) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.writeFrame(frame);
        
    }

    public static void brickletDMXWriteFrame(@Nullable ThingActions actions, int[] frame) throws TinkerforgeException {
        invokeMethodOf(actions).brickletDMXWriteFrame(frame);
    }


    @Override
    @RuleAction(label = "Get DMX Mode")
    public @ActionOutput(name = "dmxMode", type="int")
           Map<String, Object> brickletDMXGetDMXMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getDMXMode();
        
        result.put("dmxMode", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetDMXMode(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetDMXMode();
    }


    @Override
    @RuleAction(label = "Read Frame")
    public @ActionOutput(name = "frame", type="int[]")
           @ActionOutput(name = "frameNumber", type="long")
           Map<String, Object> brickletDMXReadFrame(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDMX.ReadFrame value = dev.readFrame();
        
        result.put("frame", value.frame);
        result.put("frameNumber", value.frameNumber);
        return result;
    }

    public static Map<String, Object> brickletDMXReadFrame(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXReadFrame();
    }


    @Override
    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "frameDuration", type="int")
           Map<String, Object> brickletDMXGetFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getFrameDuration();
        
        result.put("frameDuration", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetFrameDuration();
    }


    @Override
    @RuleAction(label = "Get Frame Error Count")
    public @ActionOutput(name = "overrunErrorCount", type="long")
           @ActionOutput(name = "framingErrorCount", type="long")
           Map<String, Object> brickletDMXGetFrameErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDMX.FrameErrorCount value = dev.getFrameErrorCount();
        
        result.put("overrunErrorCount", value.overrunErrorCount);
        result.put("framingErrorCount", value.framingErrorCount);
        return result;
    }

    public static Map<String, Object> brickletDMXGetFrameErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetFrameErrorCount();
    }


    @Override
    @RuleAction(label = "Get Communication LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDMXGetCommunicationLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getCommunicationLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetCommunicationLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetCommunicationLEDConfig();
    }


    @Override
    @RuleAction(label = "Get Error LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDMXGetErrorLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getErrorLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetErrorLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetErrorLEDConfig();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletDMXReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletDMXReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletDMXGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDMXGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDMXGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletDMXGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDMX.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletDMXGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletDMXReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletDMXReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletDMXReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDMXGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDMXWrapper dev = ((@Nullable BrickletDMXWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDMX.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDMXGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDMXGetIdentity();
    }
}
