/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.library.types.StringType;

import com.tinkerforge.BrickletDMX;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the DMX Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletDMXWrapper extends BrickletDMX implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletDMX.DEVICE_DISPLAY_NAME, "brickletdmx", BrickletDMX.DEVICE_IDENTIFIER, BrickletDMXWrapper.class, BrickletDMXActions.class, "2.0.0", true);

    public BrickletDMXWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletDMXWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletDMXConfig cfg = (BrickletDMXConfig) config.as(BrickletDMXConfig.class);
        if(cfg.dmxMode == 0) {
this.addFrameStartedListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletDMXFrameStarted", transformBrickletDMXFrameStartedCallback0(cfg));}}, this::removeFrameStartedListener));}
		if(cfg.dmxMode == 1) {
this.addFrameAvailableListener(this.reg((frameNumber) -> {if(true) {triggerChannelFn.accept("BrickletDMXFrameAvailable", transformBrickletDMXFrameAvailableCallback0(frameNumber, cfg));}}, this::removeFrameAvailableListener));}
		this.addFrameErrorCountListener(this.reg((overrunErrorCount, framingErrorCount) -> {if(true) {updateStateFn.accept("BrickletDMXOverrunErrorCount", transformBrickletDMXOverrunErrorCountCallback0(overrunErrorCount, framingErrorCount, cfg));}}, this::removeFrameErrorCountListener));
		this.addFrameErrorCountListener(this.reg((overrunErrorCount, framingErrorCount) -> {if(true) {updateStateFn.accept("BrickletDMXFramingErrorCount", transformBrickletDMXFramingErrorCountCallback0(overrunErrorCount, framingErrorCount, cfg));}}, this::removeFrameErrorCountListener));
        this.setDMXMode(cfg.dmxMode);
		        this.setCommunicationLEDConfig(cfg.communicationLEDConfig);
		        this.setErrorLEDConfig(cfg.errorLEDConfig);
		        if (cfg.frameDuration >= 0) {
		            this.setFrameDuration(cfg.frameDuration);
		            this.setFrameCallbackConfig(true, true, false, true);
		        } else {
		            this.setFrameCallbackConfig(false, true, false, true);
		        }
		this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        BrickletDMXConfig cfg = (BrickletDMXConfig) config.as(BrickletDMXConfig.class);
List<String> result = new ArrayList<String>();
        if (cfg.dmxMode == 0) {
                result.add("BrickletDMXFrameStarted");
            }
		if (cfg.dmxMode == 1) {
                result.add("BrickletDMXFrameAvailable");
            }
		result.add("BrickletDMXOverrunErrorCount");
		result.add("BrickletDMXFramingErrorCount");
		if (cfg.errorLEDConfig == 3) {
                result.add("BrickletDMXClearErrorLED");
            }
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletDMXOverrunErrorCount":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletDMXOverrunErrorCount"), "Overrun Error Count", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDMXOverrunErrorCount")).withDescription("The number of occurred overrun errors")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletDMXFramingErrorCount":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletDMXFramingErrorCount"), "Framing Error Count", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDMXFramingErrorCount")).withDescription("The number of occurred framing errors")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletDMXClearErrorLED":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletDMXClearErrorLED"), "Clear Error LED", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDMXClearErrorLED")).withDescription("Clears the error LED.")
.withCommandDescription(CommandDescriptionBuilder.create().withCommandOption(new CommandOption("CLEAR", "Clear")).build()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge DMX Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("DMX master and slave").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletDMXFrameStarted", new ChannelTypeUID("system", "trigger")).withLabel("Frame Started").withDescription("This channel is triggered directly after a new frame render is started. You should send the data for the next frame directly after this listener was triggered.").build(), new ChannelDefinitionBuilder("BrickletDMXFrameAvailable", new ChannelTypeUID("system", "trigger")).withLabel("Frame Available").withDescription("This channel is triggered in slave mode when a new frame was received from the DMX master and it can be read out. You have to read the frame before the master has written the next frame using the readFrame action.").build(), new ChannelDefinitionBuilder("BrickletDMXOverrunErrorCount", new ChannelTypeUID("tinkerforge", "BrickletDMXOverrunErrorCount")).withLabel("Overrun Error Count").build(), new ChannelDefinitionBuilder("BrickletDMXFramingErrorCount", new ChannelTypeUID("tinkerforge", "BrickletDMXFramingErrorCount")).withLabel("Framing Error Count").build(), new ChannelDefinitionBuilder("BrickletDMXClearErrorLED", new ChannelTypeUID("tinkerforge", "BrickletDMXClearErrorLED")).withLabel("Clear Error LED").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletdmx":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("dmxMode", Type.INTEGER).withDefault("0").withDescription("Sets the DMX mode to either master or slave.").withLabel("DMX Mode").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Master"), new ParameterOption("1", "Slave"))).build(), ConfigDescriptionParameterBuilder.create("communicationLEDConfig", Type.INTEGER).withDefault("3").withDescription("By default the LED shows communication traffic, it flickers once for every 10 received data packets. You can also turn the LED permanently on/off or show a heartbeat. If the Bricklet is in bootloader mode, the LED is off.").withLabel("Communication LED").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Communication"))).build(), ConfigDescriptionParameterBuilder.create("errorLEDConfig", Type.INTEGER).withDefault("3").withDescription("By default the error LED turns on if there is any error (see Frame Error Count Channel). If you call this function with the Show-Error option again, the LED will turn off until the next error occurs. You can also turn the LED permanently on/off or show a heartbeat. If the Bricklet is in bootloader mode, the LED is off.").withLabel("Error LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Error"))).build(), ConfigDescriptionParameterBuilder.create("frameDuration", Type.INTEGER).withDefault("-1").withDescription("The duration of a frame.<br/><br/>Example: If you want to achieve 20 frames per second, you should set the frame duration to 50ms (50ms * 20 = 1 second). If you always want to send a frame as fast as possible you can set this value to 0. To disable the frame started channel set this value to -1. This setting is only used in master mode.").withLabel("Frame Duration").withMinimum(BigDecimal.valueOf(-1)).withMaximum(BigDecimal.valueOf(65535)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletDMXOverrunErrorCount":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletDMXFramingErrorCount":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletDMXClearErrorLED":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletDMXConfig cfg = (BrickletDMXConfig) config.as(BrickletDMXConfig.class);
switch(channel) {
            case "BrickletDMXFrameStarted":
               return;
            case "BrickletDMXFrameAvailable":
               return;
            case "BrickletDMXOverrunErrorCount":
                   {
            BrickletDMX.FrameErrorCount value = this.getFrameErrorCount();
            if(true)
                updateStateFn.accept(channel, transformBrickletDMXOverrunErrorCountGetter0(value, cfg));
        }
                   return;
            case "BrickletDMXFramingErrorCount":
                   {
            BrickletDMX.FrameErrorCount value = this.getFrameErrorCount();
            if(true)
                updateStateFn.accept(channel, transformBrickletDMXFramingErrorCountGetter0(value, cfg));
        }
                   return;
            case "BrickletDMXClearErrorLED":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletDMXClearErrorLED":
                if (command instanceof StringType) {
                    this.setErrorLEDConfig(3);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletDMXFrameStartedCallback0(BrickletDMXConfig cfg) {
        return "";
    }
	    private String transformBrickletDMXFrameAvailableCallback0(long frameNumber, BrickletDMXConfig cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletDMXOverrunErrorCountCallback0(long overrunErrorCount, long framingErrorCount, BrickletDMXConfig cfg) {
        return new DecimalType(overrunErrorCount);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletDMXFramingErrorCountCallback0(long overrunErrorCount, long framingErrorCount, BrickletDMXConfig cfg) {
        return new DecimalType(framingErrorCount);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletDMXOverrunErrorCountGetter0(BrickletDMX.FrameErrorCount value, BrickletDMXConfig cfg) {
        return new DecimalType(value.overrunErrorCount);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletDMXFramingErrorCountGetter0(BrickletDMX.FrameErrorCount value, BrickletDMXConfig cfg) {
        return new DecimalType(value.framingErrorCount);
    }

}