/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletEnergyMonitor;

/**
 * Passes through actions of the Energy Monitor Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletEnergyMonitorActions implements ThingActions, IBrickletEnergyMonitorActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletEnergyMonitorActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletEnergyMonitorActions.class.getName())) {
            if (actions instanceof IBrickletEnergyMonitorActions) {
                return (IBrickletEnergyMonitorActions) actions;
            } else {
                return (IBrickletEnergyMonitorActions) Proxy.newProxyInstance(IBrickletEnergyMonitorActions.class.getClassLoader(),
                        new Class[] { IBrickletEnergyMonitorActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletEnergyMonitorActions");
    }

    
    @Override
    @RuleAction(label = "Get Energy Data")
    public @ActionOutput(name = "voltage", type="int")
           @ActionOutput(name = "current", type="int")
           @ActionOutput(name = "energy", type="int")
           @ActionOutput(name = "realPower", type="int")
           @ActionOutput(name = "apparentPower", type="int")
           @ActionOutput(name = "reactivePower", type="int")
           @ActionOutput(name = "powerFactor", type="int")
           @ActionOutput(name = "frequency", type="int")
           Map<String, Object> brickletEnergyMonitorGetEnergyData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletEnergyMonitor.EnergyData value = dev.getEnergyData();
        
        result.put("voltage", value.voltage);
        result.put("current", value.current);
        result.put("energy", value.energy);
        result.put("realPower", value.realPower);
        result.put("apparentPower", value.apparentPower);
        result.put("reactivePower", value.reactivePower);
        result.put("powerFactor", value.powerFactor);
        result.put("frequency", value.frequency);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetEnergyData(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetEnergyData();
    }

    @Override
    @RuleAction(label = "Reset Energy")
    public void brickletEnergyMonitorResetEnergy(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.resetEnergy();
        
    }

    public static void brickletEnergyMonitorResetEnergy(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletEnergyMonitorResetEnergy();
    }


    @Override
    @RuleAction(label = "Get Waveform")
    public @ActionOutput(name = "waveform", type="int[]")
           Map<String, Object> brickletEnergyMonitorGetWaveform(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int[] value = dev.getWaveform();
        
        result.put("waveform", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetWaveform(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetWaveform();
    }


    @Override
    @RuleAction(label = "Get Transformer Status")
    public @ActionOutput(name = "voltageTransformerConnected", type="boolean")
           @ActionOutput(name = "currentTransformerConnected", type="boolean")
           Map<String, Object> brickletEnergyMonitorGetTransformerStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletEnergyMonitor.TransformerStatus value = dev.getTransformerStatus();
        
        result.put("voltageTransformerConnected", value.voltageTransformerConnected);
        result.put("currentTransformerConnected", value.currentTransformerConnected);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetTransformerStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetTransformerStatus();
    }


    @Override
    @RuleAction(label = "Get Transformer Calibration")
    public @ActionOutput(name = "voltageRatio", type="int")
           @ActionOutput(name = "currentRatio", type="int")
           @ActionOutput(name = "phaseShift", type="int")
           Map<String, Object> brickletEnergyMonitorGetTransformerCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletEnergyMonitor.TransformerCalibration value = dev.getTransformerCalibration();
        
        result.put("voltageRatio", value.voltageRatio);
        result.put("currentRatio", value.currentRatio);
        result.put("phaseShift", value.phaseShift);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetTransformerCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetTransformerCalibration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletEnergyMonitorReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletEnergyMonitorGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletEnergyMonitorGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletEnergyMonitorGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletEnergyMonitor.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletEnergyMonitorReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletEnergyMonitorReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletEnergyMonitorReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletEnergyMonitorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletEnergyMonitorWrapper dev = ((@Nullable BrickletEnergyMonitorWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletEnergyMonitor.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletEnergyMonitorGetIdentity();
    }
}
