/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OpenClosedType;

import com.tinkerforge.BrickletIndustrialDigitalIn4V2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Industrial Digital In 4 Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDigitalIn4V2Wrapper extends BrickletIndustrialDigitalIn4V2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDigitalIn4V2.DEVICE_DISPLAY_NAME, "brickletindustrialdigitalin4v2", BrickletIndustrialDigitalIn4V2.DEVICE_IDENTIFIER, BrickletIndustrialDigitalIn4V2Wrapper.class, BrickletIndustrialDigitalIn4V2Actions.class, "2.0.0", true);

    public BrickletIndustrialDigitalIn4V2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDigitalIn4V2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDigitalIn4V2Config cfg = (BrickletIndustrialDigitalIn4V2Config) config.as(BrickletIndustrialDigitalIn4V2Config.class);
        this.addValueListener(this.reg((channel, changed, value) -> {if(channel == 0) {updateStateFn.accept("BrickletIndustrialDigitalIn4V2Input0", transformBrickletIndustrialDigitalIn4V2Input0Callback0(channel, changed, value, cfg));}}, this::removeValueListener));
		this.addValueListener(this.reg((channel, changed, value) -> {if(channel == 1) {updateStateFn.accept("BrickletIndustrialDigitalIn4V2Input1", transformBrickletIndustrialDigitalIn4V2Input1Callback0(channel, changed, value, cfg));}}, this::removeValueListener));
		this.addValueListener(this.reg((channel, changed, value) -> {if(channel == 2) {updateStateFn.accept("BrickletIndustrialDigitalIn4V2Input2", transformBrickletIndustrialDigitalIn4V2Input2Callback0(channel, changed, value, cfg));}}, this::removeValueListener));
		this.addValueListener(this.reg((channel, changed, value) -> {if(channel == 3) {updateStateFn.accept("BrickletIndustrialDigitalIn4V2Input3", transformBrickletIndustrialDigitalIn4V2Input3Callback0(channel, changed, value, cfg));}}, this::removeValueListener));
        
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletIndustrialDigitalIn4V2ValueConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2Input0").as(BrickletIndustrialDigitalIn4V2ValueConfig.class);
		this.setValueCallbackConfiguration(0, channelCfg.updateInterval, true);
		this.setChannelLEDConfig(0, channelCfg.ledConfig);
		}
		{
		BrickletIndustrialDigitalIn4V2ValueConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2Input1").as(BrickletIndustrialDigitalIn4V2ValueConfig.class);
		this.setValueCallbackConfiguration(1, channelCfg.updateInterval, true);
		this.setChannelLEDConfig(1, channelCfg.ledConfig);
		}
		{
		BrickletIndustrialDigitalIn4V2ValueConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2Input2").as(BrickletIndustrialDigitalIn4V2ValueConfig.class);
		this.setValueCallbackConfiguration(2, channelCfg.updateInterval, true);
		this.setChannelLEDConfig(2, channelCfg.ledConfig);
		}
		{
		BrickletIndustrialDigitalIn4V2ValueConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2Input3").as(BrickletIndustrialDigitalIn4V2ValueConfig.class);
		this.setValueCallbackConfiguration(3, channelCfg.updateInterval, true);
		this.setChannelLEDConfig(3, channelCfg.ledConfig);
		}
		{
		BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2EdgeCount0").as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
		this.setEdgeCountConfiguration(0, channelCfg.edgeType, channelCfg.debounce);
		if(channelCfg.updateInterval > 0) {
    this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {if(handler.getThing().getStatus() == ThingStatus.ONLINE) {handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount0"), RefreshType.REFRESH);}}, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
}
		}
		{
		BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2EdgeCount1").as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
		this.setEdgeCountConfiguration(1, channelCfg.edgeType, channelCfg.debounce);
		if(channelCfg.updateInterval > 0) {
    this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {if(handler.getThing().getStatus() == ThingStatus.ONLINE) {handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount1"), RefreshType.REFRESH);}}, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
}
		}
		{
		BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2EdgeCount2").as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
		this.setEdgeCountConfiguration(2, channelCfg.edgeType, channelCfg.debounce);
		if(channelCfg.updateInterval > 0) {
    this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {if(handler.getThing().getStatus() == ThingStatus.ONLINE) {handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount2"), RefreshType.REFRESH);}}, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
}
		}
		{
		BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDigitalIn4V2EdgeCount3").as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
		this.setEdgeCountConfiguration(3, channelCfg.edgeType, channelCfg.debounce);
		if(channelCfg.updateInterval > 0) {
    this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {if(handler.getThing().getStatus() == ThingStatus.ONLINE) {handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount3"), RefreshType.REFRESH);}}, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
}
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setValueCallbackConfiguration(0, 0, true);
		this.setValueCallbackConfiguration(1, 0, true);
		this.setValueCallbackConfiguration(2, 0, true);
		this.setValueCallbackConfiguration(3, 0, true);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDigitalIn4V2Input0");
		result.add("BrickletIndustrialDigitalIn4V2Input1");
		result.add("BrickletIndustrialDigitalIn4V2Input2");
		result.add("BrickletIndustrialDigitalIn4V2Input3");
		result.add("BrickletIndustrialDigitalIn4V2EdgeCount0");
		result.add("BrickletIndustrialDigitalIn4V2EdgeCount1");
		result.add("BrickletIndustrialDigitalIn4V2EdgeCount2");
		result.add("BrickletIndustrialDigitalIn4V2EdgeCount3");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletIndustrialDigitalIn4V2Value":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2Value"), "NOT USED", "Contact").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalIn4V2Value")).withDescription("The logic level that is currently measured on the channel.").build();
            case "BrickletIndustrialDigitalIn4V2EdgeCount":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2EdgeCount"), "NOT USED", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalIn4V2EdgeCount")).withDescription("The current value of the edge counter of the channel.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Digital In 4 Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("4 galvanically isolated digital inputs").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2Input0", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2Value")).withLabel("Input Value 0").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2Input1", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2Value")).withLabel("Input Value 1").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2Input2", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2Value")).withLabel("Input Value 2").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2Input3", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2Value")).withLabel("Input Value 3").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2EdgeCount0", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2EdgeCount")).withLabel("Edge Count 0").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2EdgeCount1", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2EdgeCount")).withLabel("Edge Count 1").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2EdgeCount2", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2EdgeCount")).withLabel("Edge Count 2").build(), new ChannelDefinitionBuilder("BrickletIndustrialDigitalIn4V2EdgeCount3", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalIn4V2EdgeCount")).withLabel("Edge Count 3").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdigitalin4v2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalIn4V2Value":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("100").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("ledConfig", Type.INTEGER).withDefault("3").withDescription("Each channel has a corresponding LED. You can turn the LED off, on or show a heartbeat. You can also set the LED to Channel Status. In this mode the LED is on if the channel is high and off otherwise.").withLabel("LED Configuration").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Channel Status"))).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalIn4V2EdgeCount":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("edgeType", Type.INTEGER).withDefault("0").withDescription("Configures if rising edges, falling edges or both are counted.").withLabel("Edge Type").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Rising"), new ParameterOption("1", "Falling"), new ParameterOption("2", "Both"))).build(), ConfigDescriptionParameterBuilder.create("debounce", Type.INTEGER).withDefault("100").withDescription("The debounce time is the minimum time between two count increments.").withLabel("Debounce Time").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(255)).build(), ConfigDescriptionParameterBuilder.create("resetOnRead", Type.BOOLEAN).withDefault("false").withDescription("Enabling this will reset the edge counter after openHAB reads its value. Use this if you want relative counts per update.").withLabel("Reset Edge Counter On Update").withAdvanced(true).build(), ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withLabel("Update Interval").withUnit("ms").build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDigitalIn4V2Config cfg = (BrickletIndustrialDigitalIn4V2Config) config.as(BrickletIndustrialDigitalIn4V2Config.class);
switch(channel) {
            case "BrickletIndustrialDigitalIn4V2Input0":
                   {
            boolean[] value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2Input0Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDigitalIn4V2Input1":
                   {
            boolean[] value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2Input1Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDigitalIn4V2Input2":
                   {
            boolean[] value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2Input2Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDigitalIn4V2Input3":
                   {
            boolean[] value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2Input3Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDigitalIn4V2EdgeCount0": {
                   BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
                   {
            long value = this.getEdgeCount(0, channelCfg.resetOnRead);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2EdgeCount0Getter0(value, cfg));
        }
                   return;
               }
            case "BrickletIndustrialDigitalIn4V2EdgeCount1": {
                   BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
                   {
            long value = this.getEdgeCount(1, channelCfg.resetOnRead);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2EdgeCount1Getter0(value, cfg));
        }
                   return;
               }
            case "BrickletIndustrialDigitalIn4V2EdgeCount2": {
                   BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
                   {
            long value = this.getEdgeCount(2, channelCfg.resetOnRead);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2EdgeCount2Getter0(value, cfg));
        }
                   return;
               }
            case "BrickletIndustrialDigitalIn4V2EdgeCount3": {
                   BrickletIndustrialDigitalIn4V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIndustrialDigitalIn4V2EdgeCountConfig.class);
                   {
            long value = this.getEdgeCount(3, channelCfg.resetOnRead);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDigitalIn4V2EdgeCount3Getter0(value, cfg));
        }
                   return;
               }
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input0Callback0(int channel, boolean changed, boolean value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input1Callback0(int channel, boolean changed, boolean value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input2Callback0(int channel, boolean changed, boolean value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input3Callback0(int channel, boolean changed, boolean value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input0Getter0(boolean[] value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value[0] ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input1Getter0(boolean[] value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value[1] ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input2Getter0(boolean[] value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value[2] ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2Input3Getter0(boolean[] value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return value[3] ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2EdgeCount0Getter0(long value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return new DecimalType(value);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2EdgeCount1Getter0(long value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return new DecimalType(value);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2EdgeCount2Getter0(long value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return new DecimalType(value);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalIn4V2EdgeCount3Getter0(long value, BrickletIndustrialDigitalIn4V2Config cfg) {
        return new DecimalType(value);
    }

}