/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialPTC;

/**
 * Passes through actions of the Industrial PTC Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialPTCActions implements ThingActions, IBrickletIndustrialPTCActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletIndustrialPTCActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletIndustrialPTCActions.class.getName())) {
            if (actions instanceof IBrickletIndustrialPTCActions) {
                return (IBrickletIndustrialPTCActions) actions;
            } else {
                return (IBrickletIndustrialPTCActions) Proxy.newProxyInstance(IBrickletIndustrialPTCActions.class.getClassLoader(),
                        new Class[] { IBrickletIndustrialPTCActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletIndustrialPTCActions");
    }

    
    @Override
    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialPTCGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetTemperature();
    }


    @Override
    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type="boolean")
           Map<String, Object> brickletIndustrialPTCIsSensorConnected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean value = dev.isSensorConnected();
        
        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCIsSensorConnected(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCIsSensorConnected();
    }


    @Override
    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIndustrialPTCGetWireMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getWireMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetWireMode(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetWireMode();
    }


    @Override
    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type="int")
           Map<String, Object> brickletIndustrialPTCGetResistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getResistance();
        
        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetResistance(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetResistance();
    }


    @Override
    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletIndustrialPTCGetNoiseRejectionFilter(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getNoiseRejectionFilter();
        
        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetNoiseRejectionFilter(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetNoiseRejectionFilter();
    }


    @Override
    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthResistance", type="int")
           @ActionOutput(name = "movingAverageLengthTemperature", type="int")
           Map<String, Object> brickletIndustrialPTCGetMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialPTC.MovingAverageConfiguration value = dev.getMovingAverageConfiguration();
        
        result.put("movingAverageLengthResistance", value.movingAverageLengthResistance);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetMovingAverageConfiguration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIndustrialPTCReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialPTCGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialPTCGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIndustrialPTCGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialPTC.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletIndustrialPTCReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletIndustrialPTCReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletIndustrialPTCReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialPTCGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialPTCWrapper dev = ((@Nullable BrickletIndustrialPTCWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialPTC.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialPTCGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialPTCGetIdentity();
    }
}
