/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OpenClosedType;

import com.tinkerforge.BrickletIndustrialPTC;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Industrial PTC Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialPTCWrapper extends BrickletIndustrialPTC implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialPTC.DEVICE_DISPLAY_NAME, "brickletindustrialptc", BrickletIndustrialPTC.DEVICE_IDENTIFIER, BrickletIndustrialPTCWrapper.class, BrickletIndustrialPTCActions.class, "2.0.0", true);

    public BrickletIndustrialPTCWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialPTCWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialPTCConfig cfg = (BrickletIndustrialPTCConfig) config.as(BrickletIndustrialPTCConfig.class);
        this.addTemperatureListener(this.reg((temperature) -> {if(true) {updateStateFn.accept("BrickletIndustrialPTCTemperature", transformBrickletIndustrialPTCTemperatureCallback0(temperature, cfg));}}, this::removeTemperatureListener));
		this.addResistanceListener(this.reg((resistance) -> {if(true) {updateStateFn.accept("BrickletIndustrialPTCResistance", transformBrickletIndustrialPTCResistanceCallback0(resistance, cfg));}}, this::removeResistanceListener));
		this.addSensorConnectedListener(this.reg((connected) -> {if(true) {updateStateFn.accept("BrickletIndustrialPTCSensorConnected", transformBrickletIndustrialPTCSensorConnectedCallback0(connected, cfg));}}, this::removeSensorConnectedListener));
        this.setWireMode(cfg.wireMode);
		this.setMovingAverageConfiguration(1, cfg.temperatureMovingAverageLength);
		this.setNoiseRejectionFilter(cfg.noiseRejectionFilterFrequency);
		this.setSensorConnectedCallbackConfiguration(true);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletIndustrialPTCTemperatureConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialPTCTemperature").as(BrickletIndustrialPTCTemperatureConfig.class);
		this.setTemperatureCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
		}
		{
		BrickletIndustrialPTCResistanceConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialPTCResistance").as(BrickletIndustrialPTCResistanceConfig.class);
		this.setResistanceCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setTemperatureCallbackConfiguration(0, true, 'x', 0, 0);
		this.setResistanceCallbackConfiguration(0, true, 'x', 0, 0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialPTCTemperature");
		result.add("BrickletIndustrialPTCResistance");
		result.add("BrickletIndustrialPTCSensorConnected");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletIndustrialPTCTemperature":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCTemperature"), "Temperature", "Number:Temperature").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialPTCTemperature")).withDescription("Temperature of the connected sensor")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-246)).withMaximum(BigDecimal.valueOf(849)).withStep(BigDecimal.valueOf(0.01)).withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletIndustrialPTCResistance":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCResistance"), "Resistance", "Number:ElectricResistance").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialPTCResistance")).withDescription("The value as measured by the MAX31865 precision delta-sigma ADC. Configure the thing to Pt 100 or Pt 1000, to make sure the conversion is correct.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletIndustrialPTCSensorConnected":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCSensorConnected"), "Sensor Connected", "Contact").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialPTCSensorConnected")).withDescription("Indicates if the sensor is connected correctly. If this is disabled, there is either no Pt100 or Pt1000 sensor connected, the sensor is connected incorrectly or the sensor itself is faulty.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial PTC Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Reads temperatures from Pt100 und Pt1000 sensors").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIndustrialPTCTemperature", new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCTemperature")).withLabel("Temperature").build(), new ChannelDefinitionBuilder("BrickletIndustrialPTCResistance", new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCResistance")).withLabel("Resistance").build(), new ChannelDefinitionBuilder("BrickletIndustrialPTCSensorConnected", new ChannelTypeUID("tinkerforge", "BrickletIndustrialPTCSensorConnected")).withLabel("Sensor Connected").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialptc":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("wireMode", Type.INTEGER).withDefault("2").withDescription("The wire mode of the sensor. Possible values are 2, 3 and 4 which correspond to 2-, 3- and 4-wire sensors. The value has to match the jumper configuration on the Bricklet.").withLabel("Wire Mode").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("2", "2-wire"), new ParameterOption("3", "3-wire"), new ParameterOption("4", "4-wire"))).build(), ConfigDescriptionParameterBuilder.create("temperatureMovingAverageLength", Type.INTEGER).withDefault("40").withDescription("Setting the length to 1 will turn the averaging off. With less averaging, there is more noise on the data.\n\nNew data is gathered every 20ms. With a moving average of length 1000 the resulting averaging window has a length of 20s. If you want to do long term measurements the longest moving average will give the cleanest results.").withLabel("Temperature Moving Average Length").withMinimum(BigDecimal.valueOf(1)).withMaximum(BigDecimal.valueOf(1000)).build(), ConfigDescriptionParameterBuilder.create("noiseRejectionFilterFrequency", Type.INTEGER).withDefault("0").withDescription("Sets the noise rejection filter to either 50 Hz or 60 Hz. Noise from 50 Hz or 60 Hz power sources (including harmonics of the AC power's fundamental frequency) is attenuated by 82dB").withLabel("Noise Rejection Filter Frequency").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "50Hz"), new ParameterOption("1", "60Hz"))).build(), ConfigDescriptionParameterBuilder.create("sensorType", Type.INTEGER).withDefault("100").withDescription("The type of the sensor. The value has to match the jumper configuration on the Bricklet or else the calculated resistance will not be correct.").withLabel("Sensor Type").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("100", "Pt100"), new ParameterOption("1000", "Pt1000"))).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletIndustrialPTCTemperature":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialPTCResistance":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialPTCSensorConnected":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialPTCConfig cfg = (BrickletIndustrialPTCConfig) config.as(BrickletIndustrialPTCConfig.class);
switch(channel) {
            case "BrickletIndustrialPTCTemperature":
                   {
            int value = this.getTemperature();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialPTCTemperatureGetter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialPTCResistance":
                   {
            int value = this.getResistance();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialPTCResistanceGetter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialPTCSensorConnected":
                   {
            boolean value = this.isSensorConnected();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialPTCSensorConnectedGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCTemperatureCallback0(int temperature, BrickletIndustrialPTCConfig cfg) {
        return new QuantityType<>(temperature / 100.0, SIUnits.CELSIUS);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCResistanceCallback0(int resistance, BrickletIndustrialPTCConfig cfg) {
        return new QuantityType<>(resistance / (32768 / cfg.sensorType == 100 ? 390 : 3900), SmartHomeUnits.OHM);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCSensorConnectedCallback0(boolean connected, BrickletIndustrialPTCConfig cfg) {
        return connected ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCTemperatureGetter0(int value, BrickletIndustrialPTCConfig cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.CELSIUS);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCResistanceGetter0(int value, BrickletIndustrialPTCConfig cfg) {
        return new QuantityType<>(value / (32768 / cfg.sensorType == 100 ? 390 : 3900), SmartHomeUnits.OHM);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialPTCSensorConnectedGetter0(boolean value, BrickletIndustrialPTCConfig cfg) {
        return value ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }

}