/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.StringType;

import com.tinkerforge.BrickletIsolator;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Isolator Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIsolatorWrapper extends BrickletIsolator implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIsolator.DEVICE_DISPLAY_NAME, "brickletisolator", BrickletIsolator.DEVICE_IDENTIFIER, BrickletIsolatorWrapper.class, BrickletIsolatorActions.class, "2.0.2", true);

    public BrickletIsolatorWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletIsolatorWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIsolatorConfig cfg = (BrickletIsolatorConfig) config.as(BrickletIsolatorConfig.class);
        this.addStatisticsListener(this.reg((messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID) -> {if(true) {updateStateFn.accept("BrickletIsolatorMessagesFromBrick", transformBrickletIsolatorMessagesFromBrickCallback0(messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID, cfg));}}, this::removeStatisticsListener));
		this.addStatisticsListener(this.reg((messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID) -> {if(true) {updateStateFn.accept("BrickletIsolatorMessagesFromBricklet", transformBrickletIsolatorMessagesFromBrickletCallback0(messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID, cfg));}}, this::removeStatisticsListener));
		this.addStatisticsListener(this.reg((messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID) -> {if(true) {updateStateFn.accept("BrickletIsolatorConnectedBrickletDeviceName", transformBrickletIsolatorConnectedBrickletDeviceNameCallback0(messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID, cfg));}}, this::removeStatisticsListener));
		this.addStatisticsListener(this.reg((messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID) -> {if(true) {updateStateFn.accept("BrickletIsolatorConnectedBrickletUID", transformBrickletIsolatorConnectedBrickletUIDCallback0(messagesFromBrick, messagesFromBricklet, connectedBrickletDeviceIdentifier, connectedBrickletUID, cfg));}}, this::removeStatisticsListener));
        
		        this.setSPITFPBaudrateConfig(cfg.spitfpEnableDynamicBaudrate, cfg.spitfpMinimumDynamicBaudrate);
		        this.setSPITFPBaudrate(cfg.spitfpBaudrate);
		        this.setStatisticsCallbackConfiguration(cfg.statisticsUpdateInterval, true);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletIsolatorMessagesFromBrick");
		result.add("BrickletIsolatorMessagesFromBricklet");
		result.add("BrickletIsolatorConnectedBrickletDeviceName");
		result.add("BrickletIsolatorConnectedBrickletUID");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletIsolatorMessagesFromBrick":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIsolatorMessagesFromBrick"), "Messages From Brick", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIsolatorMessagesFromBrick")).withDescription("Count of messages passed through the Isolator from the controlling Brick.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletIsolatorMessagesFromBricklet":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIsolatorMessagesFromBricklet"), "Messages From Bricköet", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIsolatorMessagesFromBricklet")).withDescription("Count of messages passed through the Isolator from the isolated Bricklet.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletIsolatorConnectedBrickletDeviceName":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIsolatorConnectedBrickletDeviceName"), "Bricklet Type", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIsolatorConnectedBrickletDeviceName")).withDescription("Device Name of the isolated Bricklet.").build();
            case "BrickletIsolatorConnectedBrickletUID":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIsolatorConnectedBrickletUID"), "Bricklet UID", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIsolatorConnectedBrickletUID")).withDescription("UID of the isolated Bricklet.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Isolator Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Galvanically isolates any Bricklet from any Brick").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIsolatorMessagesFromBrick", new ChannelTypeUID("tinkerforge", "BrickletIsolatorMessagesFromBrick")).withLabel("Messages From Brick").build(), new ChannelDefinitionBuilder("BrickletIsolatorMessagesFromBricklet", new ChannelTypeUID("tinkerforge", "BrickletIsolatorMessagesFromBricklet")).withLabel("Messages From Bricköet").build(), new ChannelDefinitionBuilder("BrickletIsolatorConnectedBrickletDeviceName", new ChannelTypeUID("tinkerforge", "BrickletIsolatorConnectedBrickletDeviceName")).withLabel("Bricklet Type").build(), new ChannelDefinitionBuilder("BrickletIsolatorConnectedBrickletUID", new ChannelTypeUID("tinkerforge", "BrickletIsolatorConnectedBrickletUID")).withLabel("Bricklet UID").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletisolator":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("spitfpEnableDynamicBaudrate", Type.BOOLEAN).withDefault("true").withDescription("The SPITF protocol can be used with a dynamic baudrate. If the dynamic baudrate is enabled, the Brick will try to adapt the baudrate for the communication between Bricks and Bricklets according to the amount of data that is transferred.<br/><br/>The baudrate for communication config between Brick and Isolator Bricklet can be set through the configuration of the Brick.<br/><br/>The baudrate will be increased exponentially if lots of data is sent/received and decreased linearly if little data is sent/received.<br/><br/>This lowers the baudrate in applications where little data is transferred (e.g. a weather station) and increases the robustness. If there is lots of data to transfer (e.g. Thermal Imaging Bricklet) it automatically increases the baudrate as needed.<br/><br/>In cases where some data has to transferred as fast as possible every few seconds (e.g. RS485 Bricklet with a high baudrate but small payload) you may want to turn the dynamic baudrate off to get the highest possible performance.<br/><br/>The maximum value of the baudrate can be set per port. If the dynamic baudrate is disabled, the maximum baudrate will be used statically.").withLabel("SPITFP Dynamic Baudrate").withAdvanced(true).build(), ConfigDescriptionParameterBuilder.create("spitfpMinimumDynamicBaudrate", Type.INTEGER).withDefault("400000").withDescription("See SPITFP Dynamic Baudrate").withLabel("SPITFP Minimum Dynamic Baudrate").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("spitfpBaudrate", Type.INTEGER).withDefault("1400000").withDescription("The baudrate used to communicate with the Bricklet.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in your applications we recommend to not change the baudrate.").withLabel("SPITFP (Maximum) Baudrate").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("statisticsUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for all statistics data in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Statistics Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletIsolatorMessagesFromBrick":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIsolatorMessagesFromBricklet":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIsolatorConnectedBrickletDeviceName":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIsolatorConnectedBrickletUID":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIsolatorConfig cfg = (BrickletIsolatorConfig) config.as(BrickletIsolatorConfig.class);
switch(channel) {
            case "BrickletIsolatorMessagesFromBrick":
                   {
            BrickletIsolator.Statistics value = this.getStatistics();
            if(true)
                updateStateFn.accept(channel, transformBrickletIsolatorMessagesFromBrickGetter0(value, cfg));
        }
                   return;
            case "BrickletIsolatorMessagesFromBricklet":
                   {
            BrickletIsolator.Statistics value = this.getStatistics();
            if(true)
                updateStateFn.accept(channel, transformBrickletIsolatorMessagesFromBrickletGetter0(value, cfg));
        }
                   return;
            case "BrickletIsolatorConnectedBrickletDeviceName":
                   {
            BrickletIsolator.Statistics value = this.getStatistics();
            if(true)
                updateStateFn.accept(channel, transformBrickletIsolatorConnectedBrickletDeviceNameGetter0(value, cfg));
        }
                   return;
            case "BrickletIsolatorConnectedBrickletUID":
                   {
            BrickletIsolator.Statistics value = this.getStatistics();
            if(true)
                updateStateFn.accept(channel, transformBrickletIsolatorConnectedBrickletUIDGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletIsolatorMessagesFromBrickCallback0(long messagesFromBrick, long messagesFromBricklet, int connectedBrickletDeviceIdentifier, String connectedBrickletUID, BrickletIsolatorConfig cfg) {
        return new DecimalType(messagesFromBrick);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorMessagesFromBrickletCallback0(long messagesFromBrick, long messagesFromBricklet, int connectedBrickletDeviceIdentifier, String connectedBrickletUID, BrickletIsolatorConfig cfg) {
        return new DecimalType(messagesFromBricklet);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorConnectedBrickletDeviceNameCallback0(long messagesFromBrick, long messagesFromBricklet, int connectedBrickletDeviceIdentifier, String connectedBrickletUID, BrickletIsolatorConfig cfg) {
        return new StringType(Helper.getDeviceName(connectedBrickletDeviceIdentifier));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorConnectedBrickletUIDCallback0(long messagesFromBrick, long messagesFromBricklet, int connectedBrickletDeviceIdentifier, String connectedBrickletUID, BrickletIsolatorConfig cfg) {
        return new StringType(connectedBrickletUID);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorMessagesFromBrickGetter0(BrickletIsolator.Statistics value, BrickletIsolatorConfig cfg) {
        return new DecimalType(value.messagesFromBrick);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorMessagesFromBrickletGetter0(BrickletIsolator.Statistics value, BrickletIsolatorConfig cfg) {
        return new DecimalType(value.messagesFromBricklet);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorConnectedBrickletDeviceNameGetter0(BrickletIsolator.Statistics value, BrickletIsolatorConfig cfg) {
        return new StringType(Helper.getDeviceName(value.connectedBrickletDeviceIdentifier));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIsolatorConnectedBrickletUIDGetter0(BrickletIsolator.Statistics value, BrickletIsolatorConfig cfg) {
        return new StringType(value.connectedBrickletUID);
    }

}