/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletLEDStrip;

/**
 * Passes through actions of the LED Strip Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripActions implements ThingActions, IBrickletLEDStripActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletLEDStripActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletLEDStripActions.class.getName())) {
            if (actions instanceof IBrickletLEDStripActions) {
                return (IBrickletLEDStripActions) actions;
            } else {
                return (IBrickletLEDStripActions) Proxy.newProxyInstance(IBrickletLEDStripActions.class.getClassLoader(),
                        new Class[] { IBrickletLEDStripActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletLEDStripActions");
    }

        @Override
    @RuleAction(label = "Set RGB Values")
    public void brickletLEDStripSetRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length,
            @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g,
            @ActionInput(name = "b") short[] b) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setRGBValues(index, length, r, g, b);
        
    }

    public static void brickletLEDStripSetRGBValues(@Nullable ThingActions actions, int index, short length, short[] r, short[] g, short[] b) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLEDStripSetRGBValues(index, length, r, g, b);
    }


    @Override
    @RuleAction(label = "Get RGB Values")
    public @ActionOutput(name = "r", type="short[]")
           @ActionOutput(name = "g", type="short[]")
           @ActionOutput(name = "b", type="short[]")
           Map<String, Object> brickletLEDStripGetRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLEDStrip.RGBValues value = dev.getRGBValues(index, length);
        
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBValues(@Nullable ThingActions actions, int index, short length) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetRGBValues(index, length);
    }


    @Override
    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> brickletLEDStripGetFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getFrameDuration();
        
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetFrameDuration();
    }


    @Override
    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletLEDStripGetSupplyVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getSupplyVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetSupplyVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetSupplyVoltage();
    }


    @Override
    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type="long")
           Map<String, Object> brickletLEDStripGetClockFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getClockFrequency();
        
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetClockFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetClockFrequency();
    }


    @Override
    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type="int")
           Map<String, Object> brickletLEDStripGetChipType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipType();
        
        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChipType(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetChipType();
    }

    @Override
    @RuleAction(label = "Set RGBW Values")
    public void brickletLEDStripSetRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length,
            @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g,
            @ActionInput(name = "b") short[] b,
            @ActionInput(name = "w") short[] w) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setRGBWValues(index, length, r, g, b, w);
        
    }

    public static void brickletLEDStripSetRGBWValues(@Nullable ThingActions actions, int index, short length, short[] r, short[] g, short[] b, short[] w) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLEDStripSetRGBWValues(index, length, r, g, b, w);
    }


    @Override
    @RuleAction(label = "Get RGBW Values")
    public @ActionOutput(name = "r", type="short[]")
           @ActionOutput(name = "g", type="short[]")
           @ActionOutput(name = "b", type="short[]")
           @ActionOutput(name = "w", type="short[]")
           Map<String, Object> brickletLEDStripGetRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLEDStrip.RGBWValues value = dev.getRGBWValues(index, length);
        
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("w", value.w);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBWValues(@Nullable ThingActions actions, int index, short length) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetRGBWValues(index, length);
    }


    @Override
    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type="short")
           Map<String, Object> brickletLEDStripGetChannelMapping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        short value = dev.getChannelMapping();
        
        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChannelMapping(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetChannelMapping();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLEDStripGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLEDStrip.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripGetIdentity();
    }
}
