/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMoisture;

/**
 * Passes through actions of the Moisture Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMoistureActions implements ThingActions, IBrickletMoistureActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletMoistureActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletMoistureActions.class.getName())) {
            if (actions instanceof IBrickletMoistureActions) {
                return (IBrickletMoistureActions) actions;
            } else {
                return (IBrickletMoistureActions) Proxy.newProxyInstance(IBrickletMoistureActions.class.getClassLoader(),
                        new Class[] { IBrickletMoistureActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletMoistureActions");
    }

    
    @Override
    @RuleAction(label = "Get Moisture Value")
    public @ActionOutput(name = "moisture", type="int")
           Map<String, Object> brickletMoistureGetMoistureValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMoistureWrapper dev = ((@Nullable BrickletMoistureWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getMoistureValue();
        
        result.put("moisture", value);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetMoistureValue(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMoistureGetMoistureValue();
    }


    @Override
    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="short")
           Map<String, Object> brickletMoistureGetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMoistureWrapper dev = ((@Nullable BrickletMoistureWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        short value = dev.getMovingAverage();
        
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMoistureGetMovingAverage();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMoistureGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMoistureWrapper dev = ((@Nullable BrickletMoistureWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletMoisture.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMoistureGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMoistureGetIdentity();
    }
}
