/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;

import com.tinkerforge.BrickletMotorizedLinearPoti;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Motorized Linear Poti Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletMotorizedLinearPotiWrapper extends BrickletMotorizedLinearPoti implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletMotorizedLinearPoti.DEVICE_DISPLAY_NAME, "brickletmotorizedlinearpoti", BrickletMotorizedLinearPoti.DEVICE_IDENTIFIER, BrickletMotorizedLinearPotiWrapper.class, BrickletMotorizedLinearPotiActions.class, "2.0.0", true);

    public BrickletMotorizedLinearPotiWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletMotorizedLinearPotiWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletMotorizedLinearPotiConfig cfg = (BrickletMotorizedLinearPotiConfig) config.as(BrickletMotorizedLinearPotiConfig.class);
        this.addPositionListener(this.reg((position) -> {if(true) {updateStateFn.accept("BrickletMotorizedLinearPotiPosition", transformBrickletMotorizedLinearPotiPositionCallback0(position, cfg));}}, this::removePositionListener));
		this.addPositionReachedListener(this.reg((position) -> {if(true) {triggerChannelFn.accept("BrickletMotorizedLinearPotiPositionReached", transformBrickletMotorizedLinearPotiPositionReachedCallback0(position, cfg));}}, this::removePositionReachedListener));
        
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletMotorizedLinearPotiPositionConfig channelCfg = getChannelConfigFn.apply("BrickletMotorizedLinearPotiPosition").as(BrickletMotorizedLinearPotiPositionConfig.class);
		this.setPositionCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setPositionCallbackConfiguration(0, true, 'x', 0, 0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletMotorizedLinearPotiPosition");
		result.add("BrickletMotorizedLinearPotiMotorPosition");
		result.add("BrickletMotorizedLinearPotiPositionReached");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletMotorizedLinearPotiPosition":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletMotorizedLinearPotiPosition"), "Position", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletMotorizedLinearPotiPosition")).withDescription("The position of the linear potentiometer. The value is between 0 (slider down) and 100 (slider up).")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(1)).withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletMotorizedLinearPotiMotorPosition":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletMotorizedLinearPotiMotorPosition"), "Motor Position", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletMotorizedLinearPotiMotorPosition")).withDescription("The target position of the potentiometer. The motorized potentiometer will immediately start to approach the position. Depending on the chosen drive mode, the position will either be reached as fast as possible or in a slow but smooth motion.<br/><br/>The position has to be between 0 (slider down) and 100 (slider up).")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(1)).withPattern("%d").withReadOnly(false).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Motorized Linear Poti Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Motorized Linear Potentiometer").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletMotorizedLinearPotiPosition", new ChannelTypeUID("tinkerforge", "BrickletMotorizedLinearPotiPosition")).withLabel("Position").build(), new ChannelDefinitionBuilder("BrickletMotorizedLinearPotiMotorPosition", new ChannelTypeUID("tinkerforge", "BrickletMotorizedLinearPotiMotorPosition")).withLabel("Motor Position").build(), new ChannelDefinitionBuilder("BrickletMotorizedLinearPotiPositionReached", new ChannelTypeUID("system", "trigger")).withLabel("Position Reached").withDescription("This channel is triggered if a new position as set by the Motor Position Channel is reached.").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletmotorizedlinearpoti":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletMotorizedLinearPotiPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletMotorizedLinearPotiMotorPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("smoothDriveMode", Type.BOOLEAN).withDefault("false").withDescription("Depending on the chosen drive mode, the position will either be reached as fast as possible or in a slow but smooth motion.").withLabel("Smooth Drive Mode").build(), ConfigDescriptionParameterBuilder.create("holdPosition", Type.BOOLEAN).withDefault("false").withDescription("If you enable the hold position flag, the position will automatically be retained. If a user changes the position of the potentiometer, it will automatically drive back to the original set point.<br/><br/>If the hold position flag disabled, the potentiometer can be changed again by the user as soon as the set point was reached once.").withLabel("Hold Position").build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletMotorizedLinearPotiConfig cfg = (BrickletMotorizedLinearPotiConfig) config.as(BrickletMotorizedLinearPotiConfig.class);
switch(channel) {
            case "BrickletMotorizedLinearPotiPosition":
                   {
            int value = this.getPosition();
            if(true)
                updateStateFn.accept(channel, transformBrickletMotorizedLinearPotiPositionGetter0(value, cfg));
        }
                   return;
            case "BrickletMotorizedLinearPotiMotorPosition":
                   {
            BrickletMotorizedLinearPoti.MotorPosition value = this.getMotorPosition();
            if(true)
                updateStateFn.accept(channel, transformBrickletMotorizedLinearPotiMotorPositionGetter0(value, cfg));
        }
                   return;
            case "BrickletMotorizedLinearPotiPositionReached":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletMotorizedLinearPotiMotorPosition":
                if (command instanceof Number) {
                    BrickletMotorizedLinearPotiMotorPositionConfig channelCfg = channelConfig.as(BrickletMotorizedLinearPotiMotorPositionConfig.class);
Number cmd = (Number) command;
this.setMotorPosition(cmd.intValue(), channelCfg.smoothDriveMode ? 1 : 0, channelCfg.holdPosition);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletMotorizedLinearPotiPositionCallback0(int position, BrickletMotorizedLinearPotiConfig cfg) {
        return new DecimalType(position);
    }
	    private String transformBrickletMotorizedLinearPotiPositionReachedCallback0(int position, BrickletMotorizedLinearPotiConfig cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletMotorizedLinearPotiPositionGetter0(int value, BrickletMotorizedLinearPotiConfig cfg) {
        return new DecimalType(value);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletMotorizedLinearPotiMotorPositionGetter0(BrickletMotorizedLinearPoti.MotorPosition value, BrickletMotorizedLinearPotiConfig cfg) {
        return new DecimalType(value.position);
    }

}