/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletNFCRFID;

/**
 * Passes through actions of the NFC/RFID Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletNFCRFIDActions implements ThingActions, IBrickletNFCRFIDActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletNFCRFIDActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletNFCRFIDActions.class.getName())) {
            if (actions instanceof IBrickletNFCRFIDActions) {
                return (IBrickletNFCRFIDActions) actions;
            } else {
                return (IBrickletNFCRFIDActions) Proxy.newProxyInstance(IBrickletNFCRFIDActions.class.getClassLoader(),
                        new Class[] { IBrickletNFCRFIDActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletNFCRFIDActions");
    }

        @Override
    @RuleAction(label = "Request Tag ID")
    public void brickletNFCRFIDRequestTagID(
            @ActionInput(name = "tagType") short tagType) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.requestTagID(tagType);
        
    }

    public static void brickletNFCRFIDRequestTagID(@Nullable ThingActions actions, short tagType) throws TinkerforgeException {
        invokeMethodOf(actions).brickletNFCRFIDRequestTagID(tagType);
    }


    @Override
    @RuleAction(label = "Get Tag ID")
    public @ActionOutput(name = "tagType", type="short")
           @ActionOutput(name = "tidLength", type="short")
           @ActionOutput(name = "tid", type="short[]")
           Map<String, Object> brickletNFCRFIDGetTagID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletNFCRFID.TagID value = dev.getTagID();
        
        result.put("tagType", value.tagType);
        result.put("tidLength", value.tidLength);
        result.put("tid", value.tid);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetTagID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletNFCRFIDGetTagID();
    }


    @Override
    @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type="short")
           @ActionOutput(name = "idle", type="boolean")
           Map<String, Object> brickletNFCRFIDGetState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletNFCRFID.State value = dev.getState();
        
        result.put("state", value.state);
        result.put("idle", value.idle);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetState(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletNFCRFIDGetState();
    }

    @Override
    @RuleAction(label = "Authenticate Mifare Classic Page")
    public void brickletNFCRFIDAuthenticateMifareClassicPage(
            @ActionInput(name = "page") int page,
            @ActionInput(name = "keyNumber") short keyNumber,
            @ActionInput(name = "key") short[] key) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.authenticateMifareClassicPage(page, keyNumber, key);
        
    }

    public static void brickletNFCRFIDAuthenticateMifareClassicPage(@Nullable ThingActions actions, int page, short keyNumber, short[] key) throws TinkerforgeException {
        invokeMethodOf(actions).brickletNFCRFIDAuthenticateMifareClassicPage(page, keyNumber, key);
    }

    @Override
    @RuleAction(label = "Write Page")
    public void brickletNFCRFIDWritePage(
            @ActionInput(name = "page") int page,
            @ActionInput(name = "data") short[] data) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.writePage(page, data);
        
    }

    public static void brickletNFCRFIDWritePage(@Nullable ThingActions actions, int page, short[] data) throws TinkerforgeException {
        invokeMethodOf(actions).brickletNFCRFIDWritePage(page, data);
    }

    @Override
    @RuleAction(label = "Request Page")
    public void brickletNFCRFIDRequestPage(
            @ActionInput(name = "page") int page) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.requestPage(page);
        
    }

    public static void brickletNFCRFIDRequestPage(@Nullable ThingActions actions, int page) throws TinkerforgeException {
        invokeMethodOf(actions).brickletNFCRFIDRequestPage(page);
    }


    @Override
    @RuleAction(label = "Get Page")
    public @ActionOutput(name = "data", type="short[]")
           Map<String, Object> brickletNFCRFIDGetPage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        short[] value = dev.getPage();
        
        result.put("data", value);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetPage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletNFCRFIDGetPage();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletNFCRFIDGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletNFCRFID.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletNFCRFIDGetIdentity();
    }
}
