/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletOLED64x48;

/**
 * Passes through actions of the OLED 64x48 Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED64x48Actions implements ThingActions, IBrickletOLED64x48Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletOLED64x48Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletOLED64x48Actions.class.getName())) {
            if (actions instanceof IBrickletOLED64x48Actions) {
                return (IBrickletOLED64x48Actions) actions;
            } else {
                return (IBrickletOLED64x48Actions) Proxy.newProxyInstance(IBrickletOLED64x48Actions.class.getClassLoader(),
                        new Class[] { IBrickletOLED64x48Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletOLED64x48Actions");
    }

        @Override
    @RuleAction(label = "Write")
    public void brickletOLED64x48Write(
            @ActionInput(name = "data") short[] data) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.write(data);
        
    }

    public static void brickletOLED64x48Write(@Nullable ThingActions actions, short[] data) throws TinkerforgeException {
        invokeMethodOf(actions).brickletOLED64x48Write(data);
    }

    @Override
    @RuleAction(label = "New Window")
    public void brickletOLED64x48NewWindow(
            @ActionInput(name = "columnFrom") short columnFrom,
            @ActionInput(name = "columnTo") short columnTo,
            @ActionInput(name = "rowFrom") short rowFrom,
            @ActionInput(name = "rowTo") short rowTo) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.newWindow(columnFrom, columnTo, rowFrom, rowTo);
        
    }

    public static void brickletOLED64x48NewWindow(@Nullable ThingActions actions, short columnFrom, short columnTo, short rowFrom, short rowTo) throws TinkerforgeException {
        invokeMethodOf(actions).brickletOLED64x48NewWindow(columnFrom, columnTo, rowFrom, rowTo);
    }

    @Override
    @RuleAction(label = "Clear Display")
    public void brickletOLED64x48ClearDisplay(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.clearDisplay();
        
    }

    public static void brickletOLED64x48ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletOLED64x48ClearDisplay();
    }

    @Override
    @RuleAction(label = "Write Line")
    public void brickletOLED64x48WriteLine(
            @ActionInput(name = "line") short line,
            @ActionInput(name = "position") short position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.writeLine(line, position, text);
        
    }

    public static void brickletOLED64x48WriteLine(@Nullable ThingActions actions, short line, short position, String text) throws TinkerforgeException {
        invokeMethodOf(actions).brickletOLED64x48WriteLine(line, position, text);
    }


    @Override
    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="short")
           @ActionOutput(name = "invert", type="boolean")
           Map<String, Object> brickletOLED64x48GetDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOLED64x48.DisplayConfiguration value = dev.getDisplayConfiguration();
        
        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        return result;
    }

    public static Map<String, Object> brickletOLED64x48GetDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOLED64x48GetDisplayConfiguration();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOLED64x48GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOLED64x48Wrapper dev = ((@Nullable BrickletOLED64x48Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOLED64x48.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOLED64x48GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOLED64x48GetIdentity();
    }
}
