/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletParticulateMatter;

/**
 * Passes through actions of the Particulate Matter Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletParticulateMatterActions implements ThingActions, IBrickletParticulateMatterActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletParticulateMatterActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletParticulateMatterActions.class.getName())) {
            if (actions instanceof IBrickletParticulateMatterActions) {
                return (IBrickletParticulateMatterActions) actions;
            } else {
                return (IBrickletParticulateMatterActions) Proxy.newProxyInstance(IBrickletParticulateMatterActions.class.getClassLoader(),
                        new Class[] { IBrickletParticulateMatterActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletParticulateMatterActions");
    }

    
    @Override
    @RuleAction(label = "Get PM Concentration")
    public @ActionOutput(name = "pm10", type="int")
           @ActionOutput(name = "pm25", type="int")
           @ActionOutput(name = "pm100", type="int")
           Map<String, Object> brickletParticulateMatterGetPMConcentration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletParticulateMatter.PMConcentration value = dev.getPMConcentration();
        
        result.put("pm10", value.pm10);
        result.put("pm25", value.pm25);
        result.put("pm100", value.pm100);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetPMConcentration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetPMConcentration();
    }


    @Override
    @RuleAction(label = "Get PM Count")
    public @ActionOutput(name = "greater03um", type="int")
           @ActionOutput(name = "greater05um", type="int")
           @ActionOutput(name = "greater10um", type="int")
           @ActionOutput(name = "greater25um", type="int")
           @ActionOutput(name = "greater50um", type="int")
           @ActionOutput(name = "greater100um", type="int")
           Map<String, Object> brickletParticulateMatterGetPMCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletParticulateMatter.PMCount value = dev.getPMCount();
        
        result.put("greater03um", value.greater03um);
        result.put("greater05um", value.greater05um);
        result.put("greater10um", value.greater10um);
        result.put("greater25um", value.greater25um);
        result.put("greater50um", value.greater50um);
        result.put("greater100um", value.greater100um);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetPMCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetPMCount();
    }


    @Override
    @RuleAction(label = "Get Sensor Info")
    public @ActionOutput(name = "sensorVersion", type="int")
           @ActionOutput(name = "lastErrorCode", type="int")
           @ActionOutput(name = "framingErrorCount", type="int")
           @ActionOutput(name = "checksumErrorCount", type="int")
           Map<String, Object> brickletParticulateMatterGetSensorInfo(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletParticulateMatter.SensorInfo value = dev.getSensorInfo();
        
        result.put("sensorVersion", value.sensorVersion);
        result.put("lastErrorCode", value.lastErrorCode);
        result.put("framingErrorCount", value.framingErrorCount);
        result.put("checksumErrorCount", value.checksumErrorCount);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetSensorInfo(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetSensorInfo();
    }

    @Override
    @RuleAction(label = "Set Enable")
    public void brickletParticulateMatterSetEnable(
            @ActionInput(name = "enable") boolean enable) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setEnable(enable);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletParticulateMatterSensorEnabled")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletParticulateMatterSensorEnabled"), RefreshType.REFRESH);
}
    }

    public static void brickletParticulateMatterSetEnable(@Nullable ThingActions actions, boolean enable) throws TinkerforgeException {
        invokeMethodOf(actions).brickletParticulateMatterSetEnable(enable);
    }


    @Override
    @RuleAction(label = "Get Enable")
    public @ActionOutput(name = "enable", type="boolean")
           Map<String, Object> brickletParticulateMatterGetEnable(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean value = dev.getEnable();
        
        result.put("enable", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetEnable();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletParticulateMatterReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletParticulateMatterGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletParticulateMatterGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletParticulateMatterGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletParticulateMatter.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletParticulateMatterReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletParticulateMatterReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletParticulateMatterReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletParticulateMatterGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletParticulateMatterWrapper dev = ((@Nullable BrickletParticulateMatterWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletParticulateMatter.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletParticulateMatterGetIdentity();
    }
}
