/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import java.time.ZonedDateTime;
import java.time.ZoneId;
import org.eclipse.smarthome.core.library.types.DateTimeType;

import com.tinkerforge.BrickletRealTimeClockV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Real-Time Clock Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRealTimeClockV2Wrapper extends BrickletRealTimeClockV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRealTimeClockV2.DEVICE_DISPLAY_NAME, "brickletrealtimeclockv2", BrickletRealTimeClockV2.DEVICE_IDENTIFIER, BrickletRealTimeClockV2Wrapper.class, BrickletRealTimeClockV2Actions.class, "2.0.0", true);

    public BrickletRealTimeClockV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletRealTimeClockV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRealTimeClockV2Config cfg = (BrickletRealTimeClockV2Config) config.as(BrickletRealTimeClockV2Config.class);
        this.addDateTimeListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {updateStateFn.accept("BrickletRealTimeClockV2DateTime", transformBrickletRealTimeClockV2DateTimeCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeDateTimeListener));
		this.addDateTimeListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {updateStateFn.accept("BrickletRealTimeClockV2Timestamp", transformBrickletRealTimeClockV2TimestampCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeDateTimeListener));
		this.addAlarmListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {triggerChannelFn.accept("BrickletRealTimeClockV2Alarm", transformBrickletRealTimeClockV2AlarmCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeAlarmListener));
        this.setDateTimeCallbackConfiguration(cfg.dateTimeUpdateInterval);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setDateTimeCallbackConfiguration(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletRealTimeClockV2DateTime");
		result.add("BrickletRealTimeClockV2Timestamp");
		result.add("BrickletRealTimeClockV2Alarm");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletRealTimeClockV2DateTime":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockV2DateTime"), "Date Time", "DateTime").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRealTimeClockV2DateTime")).withDescription("The real-time clock handles leap year and inserts the 29th of February accordingly. But leap seconds are not handled. The time is stored as UTC on the clock and converted into your system's timezone when accessed by OpenHAB.").build();
            case "BrickletRealTimeClockV2Timestamp":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockV2Timestamp"), "Timestamp", "Number:Time").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRealTimeClockV2Timestamp")).withDescription("the current date and the time of the real-time clock converted to seconds. The timestamp has an effective resolution of hundredths of a second.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Real-Time Clock Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Battery-backed real-time clock").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletRealTimeClockV2DateTime", new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockV2DateTime")).withLabel("Date Time").build(), new ChannelDefinitionBuilder("BrickletRealTimeClockV2Timestamp", new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockV2Timestamp")).withLabel("Timestamp").build(), new ChannelDefinitionBuilder("BrickletRealTimeClockV2Alarm", new ChannelTypeUID("system", "trigger")).withLabel("Alarm").withDescription("This listener is triggered every time the current date and time matches the configured alarm (see the setAlarm action).").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletrealtimeclockv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("dateTimeUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for the date time and timestamp in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Date Time Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletRealTimeClockV2DateTime":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletRealTimeClockV2Timestamp":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletRealTimeClockV2Config cfg = (BrickletRealTimeClockV2Config) config.as(BrickletRealTimeClockV2Config.class);
switch(channel) {
            case "BrickletRealTimeClockV2DateTime":
                   {
            BrickletRealTimeClockV2.DateTime value = this.getDateTime();
            if(true)
                updateStateFn.accept(channel, transformBrickletRealTimeClockV2DateTimeGetter0(value, cfg));
        }
                   return;
            case "BrickletRealTimeClockV2Timestamp":
                   {
            long value = this.getTimestamp();
            if(true)
                updateStateFn.accept(channel, transformBrickletRealTimeClockV2TimestampGetter0(value, cfg));
        }
                   return;
            case "BrickletRealTimeClockV2Alarm":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletRealTimeClockV2DateTime":
                if (command instanceof DateTimeType) {
                    DateTimeType cmd = (DateTimeType) command;
this.setDateTime(cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getYear(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getMonth().getValue(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getDayOfMonth(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getHour(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getMinute(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getSecond(), cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getNano() / 1000 / 1000 / 10, cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getDayOfWeek().getValue());
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of DateTimeType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockV2DateTimeCallback0(int year, int month, int day, int hour, int minute, int second, int centisecond, int weekday, long timestamp, BrickletRealTimeClockV2Config cfg) {
        return new DateTimeType(ZonedDateTime.of(year, month, day, hour, minute, second, centisecond * 10 * 1000 * 1000, ZoneId.of("UTC")).withZoneSameInstant(ZoneId.systemDefault()));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockV2TimestampCallback0(int year, int month, int day, int hour, int minute, int second, int centisecond, int weekday, long timestamp, BrickletRealTimeClockV2Config cfg) {
        return new QuantityType<>(timestamp / 1000.0, SmartHomeUnits.SECOND);
    }
	    private String transformBrickletRealTimeClockV2AlarmCallback0(int year, int month, int day, int hour, int minute, int second, int centisecond, int weekday, long timestamp, BrickletRealTimeClockV2Config cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockV2DateTimeGetter0(BrickletRealTimeClockV2.DateTime value, BrickletRealTimeClockV2Config cfg) {
        return new DateTimeType(ZonedDateTime.of(value.year, value.month, value.day, value.hour, value.minute, value.second, value.centisecond * 10 * 1000 * 1000, ZoneId.of("UTC")).withZoneSameInstant(ZoneId.systemDefault()));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockV2TimestampGetter0(long value, BrickletRealTimeClockV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.SECOND);
    }

}