/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OnOffType;

import com.tinkerforge.BrickletRemoteSwitchV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Remote Switch Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRemoteSwitchV2Wrapper extends BrickletRemoteSwitchV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRemoteSwitchV2.DEVICE_DISPLAY_NAME, "brickletremoteswitchv2", BrickletRemoteSwitchV2.DEVICE_IDENTIFIER, BrickletRemoteSwitchV2Wrapper.class, BrickletRemoteSwitchV2Actions.class, "2.0.0", true);

    public BrickletRemoteSwitchV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletRemoteSwitchV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRemoteSwitchV2Config cfg = (BrickletRemoteSwitchV2Config) config.as(BrickletRemoteSwitchV2Config.class);
        this.addSwitchingDoneListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletRemoteSwitchV2SwitchingDone", transformBrickletRemoteSwitchV2SwitchingDoneCallback0(cfg));}}, this::removeSwitchingDoneListener));
		this.addRemoteStatusAListener(this.reg((houseCode, receiverCode, switchTo, repeats) -> {if(true) {triggerChannelFn.accept("BrickletRemoteSwitchV2RemoteStatusAAvailable", transformBrickletRemoteSwitchV2RemoteStatusAAvailableCallback0(houseCode, receiverCode, switchTo, repeats, cfg));}}, this::removeRemoteStatusAListener));
		this.addRemoteStatusBListener(this.reg((address, unit, switchTo, dimValue, repeats) -> {if(true) {triggerChannelFn.accept("BrickletRemoteSwitchV2RemoteStatusBAvailable", transformBrickletRemoteSwitchV2RemoteStatusBAvailableCallback0(address, unit, switchTo, dimValue, repeats, cfg));}}, this::removeRemoteStatusBListener));
		this.addRemoteStatusCListener(this.reg((systemCode, deviceCode, switchTo, repeats) -> {if(true) {triggerChannelFn.accept("BrickletRemoteSwitchV2RemoteStatusCAvailable", transformBrickletRemoteSwitchV2RemoteStatusCAvailableCallback0(systemCode, deviceCode, switchTo, repeats, cfg));}}, this::removeRemoteStatusCListener));
        this.setRemoteConfiguration(cfg.remoteType, cfg.minimumRepeats, true);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletRemoteSwitchV2SwitchingDone");
		result.add("BrickletRemoteSwitchV2RemoteStatusAAvailable");
		result.add("BrickletRemoteSwitchV2RemoteStatusBAvailable");
		result.add("BrickletRemoteSwitchV2RemoteStatusCAvailable");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Remote Switch Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Controls remote mains switches and receives signals from remotes").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletRemoteSwitchV2SwitchingDone", new ChannelTypeUID("system", "trigger")).withLabel("Switching Done").withDescription("This channel is triggered whenever the switching state changes from busy to ready.").build(), new ChannelDefinitionBuilder("BrickletRemoteSwitchV2RemoteStatusAAvailable", new ChannelTypeUID("system", "trigger")).withLabel("Remote Status A Available").withDescription("This channel is triggered if at least the configured minimum of repeats of identical data packets for remote type A were received. You can get the house and receiver code, switch state and repeats with the getRemoteStatusA action.").build(), new ChannelDefinitionBuilder("BrickletRemoteSwitchV2RemoteStatusBAvailable", new ChannelTypeUID("system", "trigger")).withLabel("Remote Status B Available").withDescription("This channel is triggered if at least the configured minimum of repeats of identical data packets for remote type B were received. You can get the house and receiver code, switch state and repeats with the getRemoteStatusB action.").build(), new ChannelDefinitionBuilder("BrickletRemoteSwitchV2RemoteStatusCAvailable", new ChannelTypeUID("system", "trigger")).withLabel("Remote Status C Available").withDescription("This channel is triggered if at least the configured minimum of repeats of identical data packets for remote type C were received. You can get the house and receiver code, switch state and repeats with the getRemoteStatusC action.").build())).withProperties(props).buildBridge();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletremoteswitchv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("remoteType", Type.INTEGER).withDefault("0").withDescription("Type A, B or C depending on the type of remote you want to receive.").withLabel("Remote Type").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "A"), new ParameterOption("1", "B"), new ParameterOption("2", "C"))).build(), ConfigDescriptionParameterBuilder.create("minimumRepeats", Type.INTEGER).withDefault("2").withDescription("The minimum number of repeated data packets until the Remote Status channels trigger.").withLabel("Minimum Repeats").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(65535)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        switch(channel) {
            case "BrickletRemoteSwitchV2SwitchingDone":
               return;
            case "BrickletRemoteSwitchV2RemoteStatusAAvailable":
               return;
            case "BrickletRemoteSwitchV2RemoteStatusBAvailable":
               return;
            case "BrickletRemoteSwitchV2RemoteStatusCAvailable":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletRemoteSwitchV2SwitchingDoneCallback0(BrickletRemoteSwitchV2Config cfg) {
        return "";
    }
	    private String transformBrickletRemoteSwitchV2RemoteStatusAAvailableCallback0(int houseCode, int receiverCode, int switchTo, int repeats, BrickletRemoteSwitchV2Config cfg) {
        return "";
    }
	    private String transformBrickletRemoteSwitchV2RemoteStatusBAvailableCallback0(long address, int unit, int switchTo, int dimValue, int repeats, BrickletRemoteSwitchV2Config cfg) {
        return "";
    }
	    private String transformBrickletRemoteSwitchV2RemoteStatusCAvailableCallback0(char systemCode, int deviceCode, int switchTo, int repeats, BrickletRemoteSwitchV2Config cfg) {
        return "";
    }

}