/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;

import com.tinkerforge.BrickletRotaryPoti;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Rotary Poti Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRotaryPotiWrapper extends BrickletRotaryPoti implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRotaryPoti.DEVICE_DISPLAY_NAME, "brickletrotarypoti", BrickletRotaryPoti.DEVICE_IDENTIFIER, BrickletRotaryPotiWrapper.class, BrickletRotaryPotiActions.class, "2.0.0", false);

    public BrickletRotaryPotiWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletRotaryPotiWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRotaryPotiConfig cfg = (BrickletRotaryPotiConfig) config.as(BrickletRotaryPotiConfig.class);
        this.addPositionListener(this.reg((position) -> {if(true) {updateStateFn.accept("BrickletRotaryPotiPosition", transformBrickletRotaryPotiPositionCallback0(position, cfg));}}, this::removePositionListener));
		this.addAnalogValueListener(this.reg((value) -> {if(true) {updateStateFn.accept("BrickletRotaryPotiAnalogValue", transformBrickletRotaryPotiAnalogValueCallback0(value, cfg));}}, this::removeAnalogValueListener));
        
		{
		BrickletRotaryPotiPositionConfig channelCfg = getChannelConfigFn.apply("BrickletRotaryPotiPosition").as(BrickletRotaryPotiPositionConfig.class);
		this.setPositionCallbackPeriod(channelCfg.updateInterval);
		this.setPositionCallbackThreshold('x', (short)0, (short)0);
		}
		{
		BrickletRotaryPotiAnalogValueConfig channelCfg = getChannelConfigFn.apply("BrickletRotaryPotiAnalogValue").as(BrickletRotaryPotiAnalogValueConfig.class);
		this.setAnalogValueCallbackPeriod(channelCfg.updateInterval);
		this.setAnalogValueCallbackThreshold('x', (short)0, (short)0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setPositionCallbackPeriod(0);
		this.setAnalogValueCallbackPeriod(0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletRotaryPotiPosition");
		result.add("BrickletRotaryPotiAnalogValue");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletRotaryPotiPosition":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRotaryPotiPosition"), "Position", "Number:Angle").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRotaryPotiPosition")).withDescription("The position of the rotary potentiometer. The value is and between -150° (turned left) and 150° (turned right).")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-150)).withMaximum(BigDecimal.valueOf(150)).withStep(BigDecimal.valueOf(1)).withPattern("%d %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletRotaryPotiAnalogValue":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRotaryPotiAnalogValue"), "Analog Value", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRotaryPotiAnalogValue")).withDescription("<br/>Returns the value as read by a 12-bit analog-to-digital converter.<br/><br/>.. note::<br/> The value returned by :func:`Get Position` is averaged over several samples<br/> to yield less noise, while :func:`Get Analog Value` gives back raw<br/> unfiltered analog values. The only reason to use :func:`Get Analog Value` is,<br/> if you need the full resolution of the analog-to-digital converter.<br/>")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4096)).withStep(BigDecimal.valueOf(1)).withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Rotary Poti Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("300° rotary potentiometer").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletRotaryPotiPosition", new ChannelTypeUID("tinkerforge", "BrickletRotaryPotiPosition")).withLabel("Position").build(), new ChannelDefinitionBuilder("BrickletRotaryPotiAnalogValue", new ChannelTypeUID("tinkerforge", "BrickletRotaryPotiAnalogValue")).withLabel("Analog Value").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletrotarypoti":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletRotaryPotiPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletRotaryPotiAnalogValue":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletRotaryPotiConfig cfg = (BrickletRotaryPotiConfig) config.as(BrickletRotaryPotiConfig.class);
switch(channel) {
            case "BrickletRotaryPotiPosition":
                   {
            short value = this.getPosition();
            if(true)
                updateStateFn.accept(channel, transformBrickletRotaryPotiPositionGetter0(value, cfg));
        }
                   return;
            case "BrickletRotaryPotiAnalogValue":
                   {
            int value = this.getAnalogValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletRotaryPotiAnalogValueGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletRotaryPotiPositionCallback0(short position, BrickletRotaryPotiConfig cfg) {
        return new QuantityType<>(position, SmartHomeUnits.DEGREE_ANGLE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRotaryPotiAnalogValueCallback0(int value, BrickletRotaryPotiConfig cfg) {
        return new DecimalType(value);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRotaryPotiPositionGetter0(short value, BrickletRotaryPotiConfig cfg) {
        return new QuantityType<>(value, SmartHomeUnits.DEGREE_ANGLE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRotaryPotiAnalogValueGetter0(int value, BrickletRotaryPotiConfig cfg) {
        return new DecimalType(value);
    }

}