/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletThermocoupleV2;

/**
 * Passes through actions of the Thermocouple Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleV2Actions implements ThingActions, IBrickletThermocoupleV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletThermocoupleV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletThermocoupleV2Actions.class.getName())) {
            if (actions instanceof IBrickletThermocoupleV2Actions) {
                return (IBrickletThermocoupleV2Actions) actions;
            } else {
                return (IBrickletThermocoupleV2Actions) Proxy.newProxyInstance(IBrickletThermocoupleV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletThermocoupleV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletThermocoupleV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletThermocoupleV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetTemperature();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "thermocoupleType", type="int")
           @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletThermocoupleV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletThermocoupleV2.Configuration value = dev.getConfiguration();
        
        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetConfiguration();
    }


    @Override
    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type="boolean")
           @ActionOutput(name = "openCircuit", type="boolean")
           Map<String, Object> brickletThermocoupleV2GetErrorState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletThermocoupleV2.ErrorState value = dev.getErrorState();
        
        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetErrorState(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetErrorState();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletThermocoupleV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletThermocoupleV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletThermocoupleV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletThermocoupleV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletThermocoupleV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletThermocoupleV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletThermocoupleV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletThermocoupleV2Wrapper dev = ((@Nullable BrickletThermocoupleV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletThermocoupleV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletThermocoupleV2GetIdentity();
    }
}
