/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OpenClosedType;

import com.tinkerforge.BrickletThermocoupleV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Thermocouple Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletThermocoupleV2Wrapper extends BrickletThermocoupleV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletThermocoupleV2.DEVICE_DISPLAY_NAME, "brickletthermocouplev2", BrickletThermocoupleV2.DEVICE_IDENTIFIER, BrickletThermocoupleV2Wrapper.class, BrickletThermocoupleV2Actions.class, "2.0.0", true);

    public BrickletThermocoupleV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletThermocoupleV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletThermocoupleV2Config cfg = (BrickletThermocoupleV2Config) config.as(BrickletThermocoupleV2Config.class);
        this.addTemperatureListener(this.reg((temperature) -> {if(true) {updateStateFn.accept("BrickletThermocoupleV2Temperature", transformBrickletThermocoupleV2TemperatureCallback0(temperature, cfg));}}, this::removeTemperatureListener));
		this.addErrorStateListener(this.reg((overUnder, openCircuit) -> {if(true) {updateStateFn.accept("BrickletThermocoupleV2OverUnderVoltage", transformBrickletThermocoupleV2OverUnderVoltageCallback0(overUnder, openCircuit, cfg));}}, this::removeErrorStateListener));
		this.addErrorStateListener(this.reg((overUnder, openCircuit) -> {if(true) {updateStateFn.accept("BrickletThermocoupleV2OpenCircuit", transformBrickletThermocoupleV2OpenCircuitCallback0(overUnder, openCircuit, cfg));}}, this::removeErrorStateListener));
        this.setConfiguration(cfg.averageLength, cfg.thermocoupleType, cfg.frequencyFilter);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletThermocoupleV2TemperatureConfig channelCfg = getChannelConfigFn.apply("BrickletThermocoupleV2Temperature").as(BrickletThermocoupleV2TemperatureConfig.class);
		this.setTemperatureCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setTemperatureCallbackConfiguration(0, true, 'x', 0, 0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletThermocoupleV2Temperature");
		result.add("BrickletThermocoupleV2OverUnderVoltage");
		result.add("BrickletThermocoupleV2OpenCircuit");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletThermocoupleV2Temperature":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2Temperature"), "Temperature", "Number:Temperature").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletThermocoupleV2Temperature")).withDescription("The temperature of the thermocouple.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-210)).withMaximum(BigDecimal.valueOf(1800)).withStep(BigDecimal.valueOf(0.01)).withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletThermocoupleV2OverUnderVoltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2OverUnderVoltage"), "Over/Under Voltage Error", "Contact").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletThermocoupleV2OverUnderVoltage")).withDescription("Over/Under Voltage happens for voltages below 0V or above 3.3V. In this case it is very likely that your thermocouple is defective.").build();
            case "BrickletThermocoupleV2OpenCircuit":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2OpenCircuit"), "Open Circuit Error", "Contact").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletThermocoupleV2OpenCircuit")).withDescription("An Open Circuit error indicates that there is no thermocouple connected.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Thermocouple Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Measures temperature with thermocouples").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletThermocoupleV2Temperature", new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2Temperature")).withLabel("Temperature").build(), new ChannelDefinitionBuilder("BrickletThermocoupleV2OverUnderVoltage", new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2OverUnderVoltage")).withLabel("Over/Under Voltage").build(), new ChannelDefinitionBuilder("BrickletThermocoupleV2OpenCircuit", new ChannelTypeUID("tinkerforge", "BrickletThermocoupleV2OpenCircuit")).withLabel("Open Circuit").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletthermocouplev2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("averageLength", Type.INTEGER).withDefault("16").withDescription("Available averaging sizes are 1, 2, 4, 8 and 16 samples.<br/><br/>The conversion time depends on the averaging and filter configuration, it can be calculated as follows:<ul><li>60Hz: time = 82 + (samples - 1) * 16.67</li><li>50Hz: time = 98 + (samples - 1) * 20</li></ul>").withLabel("Average Length").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("1", "1"), new ParameterOption("2", "2"), new ParameterOption("4", "4"), new ParameterOption("8", "8"), new ParameterOption("16", "16"))).build(), ConfigDescriptionParameterBuilder.create("thermocoupleType", Type.INTEGER).withDefault("3").withDescription("As thermocouple type you can use B, E, J, K, N, R, S and T. If you have a different thermocouple or a custom thermocouple you can also use G8 and G32. With these types the returned value will not be in °C/100, it will be calculated by the following formulas:<ul><li>G8: value = 8 * 1.6 * 2^17 * Vin</li><li>G32: value = 32 * 1.6 * 2^17 * Vin</li></ul>where Vin is the thermocouple input voltage.").withLabel("Thermocouple Type").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "B"), new ParameterOption("1", "E"), new ParameterOption("2", "J"), new ParameterOption("3", "K"), new ParameterOption("4", "N"), new ParameterOption("5", "R"), new ParameterOption("6", "S"), new ParameterOption("7", "T"), new ParameterOption("8", "G8"), new ParameterOption("9", "G32"))).build(), ConfigDescriptionParameterBuilder.create("frequencyFilter", Type.INTEGER).withDefault("0").withDescription("The frequency filter can be either configured to 50Hz or to 60Hz. You should configure it according to your utility frequency.").withLabel("Frequency Filter").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "50Hz"), new ParameterOption("1", "60Hz"))).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletThermocoupleV2Temperature":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletThermocoupleV2OverUnderVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletThermocoupleV2OpenCircuit":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletThermocoupleV2Config cfg = (BrickletThermocoupleV2Config) config.as(BrickletThermocoupleV2Config.class);
switch(channel) {
            case "BrickletThermocoupleV2Temperature":
                   {
            int value = this.getTemperature();
            if(true)
                updateStateFn.accept(channel, transformBrickletThermocoupleV2TemperatureGetter0(value, cfg));
        }
                   return;
            case "BrickletThermocoupleV2OverUnderVoltage":
                   {
            BrickletThermocoupleV2.ErrorState value = this.getErrorState();
            if(true)
                updateStateFn.accept(channel, transformBrickletThermocoupleV2OverUnderVoltageGetter0(value, cfg));
        }
                   return;
            case "BrickletThermocoupleV2OpenCircuit":
                   {
            BrickletThermocoupleV2.ErrorState value = this.getErrorState();
            if(true)
                updateStateFn.accept(channel, transformBrickletThermocoupleV2OpenCircuitGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2TemperatureCallback0(int temperature, BrickletThermocoupleV2Config cfg) {
        return new QuantityType<>(temperature / 100.0, SIUnits.CELSIUS);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2OverUnderVoltageCallback0(boolean overUnder, boolean openCircuit, BrickletThermocoupleV2Config cfg) {
        return overUnder ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2OpenCircuitCallback0(boolean overUnder, boolean openCircuit, BrickletThermocoupleV2Config cfg) {
        return openCircuit ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2TemperatureGetter0(int value, BrickletThermocoupleV2Config cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.CELSIUS);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2OverUnderVoltageGetter0(BrickletThermocoupleV2.ErrorState value, BrickletThermocoupleV2Config cfg) {
        return value.overUnder ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletThermocoupleV2OpenCircuitGetter0(BrickletThermocoupleV2.ErrorState value, BrickletThermocoupleV2Config cfg) {
        return value.openCircuit ? OpenClosedType.CLOSED : OpenClosedType.OPEN;
    }

}