/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2021-12-16.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletVoltageCurrentV2;

/**
 * Passes through actions of the Voltage/Current Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentV2Actions implements ThingActions, IBrickletVoltageCurrentV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletVoltageCurrentV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletVoltageCurrentV2Actions.class.getName())) {
            if (actions instanceof IBrickletVoltageCurrentV2Actions) {
                return (IBrickletVoltageCurrentV2Actions) actions;
            } else {
                return (IBrickletVoltageCurrentV2Actions) Proxy.newProxyInstance(IBrickletVoltageCurrentV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletVoltageCurrentV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletVoltageCurrentV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetVoltage();
    }


    @Override
    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getCurrent();
        
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetCurrent();
    }


    @Override
    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getPower();
        
        result.put("power", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetPower(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetPower();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "voltageConversionTime", type="int")
           @ActionOutput(name = "currentConversionTime", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Configuration value = dev.getConfiguration();
        
        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetConfiguration();
    }


    @Override
    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "voltageMultiplier", type="int")
           @ActionOutput(name = "voltageDivisor", type="int")
           @ActionOutput(name = "currentMultiplier", type="int")
           @ActionOutput(name = "currentDivisor", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Calibration value = dev.getCalibration();
        
        result.put("voltageMultiplier", value.voltageMultiplier);
        result.put("voltageDivisor", value.voltageDivisor);
        result.put("currentMultiplier", value.currentMultiplier);
        result.put("currentDivisor", value.currentDivisor);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetCalibration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletVoltageCurrentV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletVoltageCurrentV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletVoltageCurrentV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletVoltageCurrentV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentV2Wrapper dev = ((@Nullable BrickletVoltageCurrentV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrentV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentV2GetIdentity();
    }
}
